'use strict';

const jwk = require('jsonwebtoken');
const jwkToPem = require('jwk-to-pem');
const request = require('request');
const { getErrorResponse, getResponse } = require('../utils/responses');

const iss = 'https://sheep-app-api.eu.auth0.com/';

module.exports = (event, context, callback, f) => {
    //Check if request contains Authorization header
    if (event.headers.Authorization) {
        // Remove 'bearer ' from token:
        const token = event.headers.Authorization.substring(7);
        // Make a request to the iss + .well-known/jwks.json URL:
        request(
            { url: `${iss}.well-known/jwks.json`, json: true },
            (error, response, body) => {
                if (error || response.statusCode !== 200) {
                    console.log('Request error:', error);
                    callback(null, getErrorResponse('Can\'t validate token'));
                }
                const keys = body;
                // Based on the JSON of `jwks` create a Pem:
                const k = keys.keys[0];
                const jwkArray = {
                    kty: k.kty,
                    n: k.n,
                    e: k.e,
                };
                const pem = jwkToPem(jwkArray);

                // Verify the token:
                jwk.verify(token, pem, { issuer: iss }, (err, decoded) => {
                    if (err) {
                        callback(null, getErrorResponse(err.message, 401));
                    } else {
                        if (decoded.iat > decoded.exp) {
                            callback(null, getErrorResponse('Token has expired', 401));
                        } else {
                            // Run function
                            f();
                        }

                    }
                });
            });
    } else {
        console.log('No authorizationToken found in the header.');
        callback(null, getErrorResponse('Missing authorization token', 401));
    }
};
