'use strict';

const AWS = require('aws-sdk');
const dynamoDb = new AWS.DynamoDB.DocumentClient();

function writeObservationRoundToDb(item) {
    const params = {
        TableName: 'observationRounds',
        Item: item
    };

    return new Promise((resolve, reject) => {
        dynamoDb.put(params, (error, result) => {
            if (error) {
                console.error(error);
                reject(error);
            }
            resolve();
        });
    });

}

function writeObservationsToDb(observations) {
    const batchRequest = {
        RequestItems: {
            "observations": observations
        }
    };

    return new Promise((resolve, reject) => {
        dynamoDb.batchWrite(batchRequest, (error, result) => {
            if (error) {
                console.error(error);
                reject(error);
            }
            resolve();
        });
    });
}

function getObservationRounds() {
    const params = {
        TableName: 'observationRounds',
    };

    return new Promise((resolve, reject) => {
        dynamoDb.scan(params, (error, result) => {
            if (error) {
                console.error(error);
                reject(error);
            }
            resolve(result.Items);
        });
    });
}

function getObservationRound(id) {
    const params = {
        TableName: 'observationRounds',
        Key: {
            id
        }
    };

    return new Promise((resolve, reject) => {
        dynamoDb.get(params, (error, result) => {
            if (error) {
                console.error(error);
                reject('unknown');
            }
            resolve(result.Item);

        });
    });
}

module.exports = {
    writeObservationRoundToDb,
    writeObservationsToDb,
    getObservationRounds,
    getObservationRound
};