'use strict';

function getObservations(data) {
    const observations = [];

    for (let i = 0; i < data.registrationPoints.length; i++) {
        const observationPoint = data.registrationPoints[i];

        for (let j = 0; j < observationPoint.observations.length; j++) {
            const observation = observationPoint.observations[j];
            observations.push(observation.data);
        }
    }
    return observations;
}

function getNumberOfSheep(groups) {
    groups = groups[0];
    let ewes = 0;
    let lamb = 0;
    const keys = Object.keys(groups.ewes);  // Assume that both lists have the same keys
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        if (groups.ewes[key] > 0) {
            ewes += groups.ewes[key];
        }
        if (groups.lamb[key] > 0) {
            lamb = groups.lamb[key]
        }
    }
    return {
        sheep: ewes + lamb,
        ewes,
        lamb
    }
}

function getDistance(lat1, lon1, lat2, lon2) {
    const pi = 0.017453292519943295;    // Math.PI / 180
    const cos = Math.cos;
    const result = 0.5 - cos((lat2 - lat1) * pi) / 2 +
        cos(lat1 * pi) * cos(lat2 * pi) *
        (1 - cos((lon2 - lon1) * pi)) / 2;

    return 12742 * Math.asin(Math.sqrt(result)); // 2 * R; R = 6371 km
}

function getWalkedDistance(file) {
    let distance = 0;
    const points = file.userTrackingPoints;
    for (let i = 0; i < points.length - 1; i += 1) {
        distance += getDistance(points[i][0], points[i][1], points[i + 1][0], points[i + 1][1]);
    }
    const formattedDistance = Math.round(distance * 1000);
    return formattedDistance + 'm';
}

function getTimeSpent(file) {
    const createdAt = file.createdTimestamp;
    const lastSaved = file.lastSavedTimestamp;
    let seconds = (lastSaved - createdAt) / 1000;
    const hours = Math.floor(seconds / 3600);
    seconds -= (hours * 3600);
    const minutes = Math.floor(seconds / 60);

    if (hours > 0) {
        return `${hours}t ${minutes}m`;
    }
    return `${minutes}m`;
}

function getStatistics(data) {
    const observations = getObservations(data);

    const res = {
        other: 0,
        predator: 0,
        reindeer: 0,
        wool: 0,
        deadSheep: 0,
        dog: 0,
        total: 0,
        ewes: 0,
        lamb: 0,
        injuredSheep: 0,
        walkedDistance: getWalkedDistance(data),
        timeSpent: getTimeSpent(data)
    };

    for (let i = 0; i < observations.length; i++) {
        const observation = observations[i];

        switch (observation.type) {
            case 'Annet':
                res.other++;
                break;
            case 'Rovdyr':
                res.predator++;
                break;
            case 'Reinsdyr':
                res.reindeer++;
                break;
            case 'Ullfunn':
                res.wool++;
                break;
            case 'Død sau':
                res.deadSheep++;
                break;
            case 'Hund':
                res.dog++;
                break;
            case 'Sau': {
                const o = getNumberOfSheep(observation.groups);
                res.total += o.sheep;
                res.ewes += o.ewes;
                res.lamb += o.lamb;
                if (observation.hasInjury === "Ja") {
                    res.injuredSheep += observation.injuries.length;
                }
                break;
            }
        }
    }
    return res;
}

module.exports.getStatistics = getStatistics;
