/* eslint-disable react/forbid-prop-types */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Swipeable from 'react-swipeable';
import { openConfirmWindow } from '../utilities/warning_window.js';

class ImageViewer extends Component {
  constructor(props) {
    super(props);
    this.state = {
      index: 0
    };
    this.nextImage = this.nextImage.bind(this);
    this.previousImage = this.previousImage.bind(this);
    this.setIndex = this.setIndex.bind(this);
    this.close = this.close.bind(this);
    this.deleteImage = this.deleteImage.bind(this);
    this.getImage = this.getImage.bind(this);
  }

  setIndex(index) {
    this.setState({
      index
    });
  }

  getImage() {
    if (this.props.images.length < 1) {
      return <p>Denne observasjonen inneholder ingen bilder</p>;
    }
    return <img src={this.props.images[this.state.index].path} alt="sheep" />;
  }

  getCircles() {
    const output = [];
    for (let i = 0; i < this.props.images.length; i += 1) {
      if (i === this.state.index) {
        const tag = <i key={i} className="fa fa-circle" />;
        output.push(tag);
      } else {
        const tag = <i key={i} className="fa fa fa-circle-thin" onClick={() => { this.setIndex(i); }} />;
        output.push(tag);
      }
    }
    return output;
  }

  previousImage() {
    let index = this.state.index - 1;
    const images = this.props.images;
    if (index < 0) {
      index = images.length - 1;
    }
    this.setState({
      index
    });
  }

  nextImage() {
    let index = this.state.index + 1;
    const images = this.props.images;
    if (index >= images.length) {
      index = 0;
    }
    this.setState({
      index
    });
  }

  close() {
    this.props.close();
  }

  deleteImage() {
    openConfirmWindow('Er du sikker på at du ønsker å slette bildet?', () => {
      this.props.onDeleteImage(this.props.openObservationIds, this.state.index);
      this.setState({
        index: 0
      });
    });
  }

  render() {
    /* eslint-disable jsx-a11y/no-static-element-interactions */
    const noImages = this.props.images.length < 1;
    const belowTwoImages = this.props.images.length < 2;
    return (
      <Swipeable onSwipedLeft={this.nextImage} onSwipedRight={this.previousImage}>
        <div className={`image-viewer${this.props.hide ? ' hide' : ''}`} >
          <i className="fa fa-remove close-image-viewer" onClick={this.close} />
          <i className={`fa fa-trash delete-image${noImages || this.props.isInForm ? ' hide' : ''}`} onClick={this.deleteImage} />
          <div className="image-container">
            {this.getImage()}
          </div>
          <div className={`buttons-container${belowTwoImages ? ' hide' : ''}`}>
            <div onClick={this.previousImage} className="fa fa-angle-left previous" />
            <div className="circles">
              {
                this.getCircles().map(circle => { return circle; })
              }
            </div>
            <div onClick={this.nextImage} className="fa fa-angle-right next" />
          </div>
        </div>
      </Swipeable>
    );
  }
}


ImageViewer.defaultProps = {
  images: [
    'http://cdn.images.express.co.uk/img/dynamic/78/590x/sheep-617128.jpg',
    'https://media.timeout.com/images/103778879/630/472/image.jpg',
    'https://animalsaustralia-api.org/cached/image/1000sheep4.jpg',
    'https://fda85f413bf08acd1c68d8df-swiftthemes.netdna-ssl.com/wp-content/uploads/2012/11/screenshot-in-android-phone-Galaxy-S2.jpeg'
  ],
  hide: false,
  isInForm: false,
  onDeleteImage: () => {}
};

if (__DEV__) {
  ImageViewer.propTypes = {
    images: PropTypes.array,
    hide: PropTypes.bool,
    isInForm: PropTypes.bool
  };
}

export default ImageViewer;

