import { ALTERNATIVES, SHEEP_COLOR, COUNTER, TEXT_FIELD } from './formElementtTypes';
import { sheepColors } from './sheepColors';
import sheepTypes from './sheepTypes';
import { baseUrl } from './NewObservationCategories';

export const EWE_ALONE = 'Søye alene';
export const LAMB_ALONE = 'Lam alene';
export const SHEEP_GROUP = 'Gruppe';
export const MULTIPLE_GROUPS = 'Flere grupper';

export const FIELD_COLOR = 'color';

export const categories = [
  EWE_ALONE,
  LAMB_ALONE,
  SHEEP_GROUP
];

export const initialState = {
  category: '',
  categoryExpanded: true,
  numberOfGroups: 2
};

const injuryCategories = [
  'Halter',
  'Hodeskade',
  'Kroppskade',
  'Blødning',
  'Annet'
];

const decisions = [
  'La leve, ta med hjem',
  'Avliv, ta med hjem',
  'Avliv, dekk til',
  'Etterlat',
  'Avventer beslutning'
];

// const injuries = {
//   Halter: [
//     'Høyre bakfot',
//     'Venstre bakfot',
//     'Bagge bakføtter',
//     'Høyre frontfot',
//     'Venstre frontfot'
//   ],
//   Hodeskade: [
//     'Høyre side',
//     'Venstre side',
//     'Begge sider'
//   ],
//   Kroppskade: [
//     'Høyre side',
//     'Venstre side',
//     'Begge sider'
//   ],
//   Blødning: [
//     'Høyre side',
//     'Venstre side',
//     'Begge sider',
//     'Hodet'
//   ]
// };

// const injuries = [
//   'Halter',
//   'Hodeskade',
//   'Kroppskade',
//   'Blødning'
// ];

const limp = [
  'Høyre bakfot',
  'Venstre bakfot',
  'Bagge bakføtter',
  'Høyre frontfot',
  'Venstre frontfot'
];

const headInjury = [
  'Høyre side',
  'Venstre side',
  'Begge sider'
];

const bodyInjury = [
  'Høyre side',
  'Venstre side',
  'Begge sider'
];

const bleeding = [
  'Høyre side',
  'Venstre side',
  'Begge sider',
  'Hodet'
];

const yesNo = [
  'Ja',
  'Nei'
];


// TODO: Move this?
export const getSheepGroup = (index = -1, data) => {
  if (data !== undefined) {
    let res = [];

    data.groups.forEach((element, dataElements) => {
      const e = [
        {
          heading: 'Antall søyer',
          formInfo: {
            type: SHEEP_COLOR,
            groupIndex: index,
            alternatives: sheepColors,
            field: 'ewes',
            key: 'ewes' + dataElements
          },
          key: dataElements
        },
        {
          heading: 'Antall lam',
          formInfo: {
            type: SHEEP_COLOR,
            groupIndex: dataElements,
            alternatives: sheepColors,
            field: 'lamb',
            key: 'lamb' + dataElements
          },
          key: dataElements
        }
      ];
      res = res.concat(e);
    });

    return res;
  }

  return [
    {
      heading: 'Antall søyer',
      formInfo: {
        type: SHEEP_COLOR,
        groupIndex: index,
        alternatives: sheepColors,
        field: 'ewes'
      },
      key: index
    },
    {
      heading: 'Antall lam',
      formInfo: {
        type: SHEEP_COLOR,
        groupIndex: index,
        alternatives: sheepColors,
        field: 'lamb'
      },
      key: index
    }
  ];
};

export function getInjuryStructure(index) {
  return [
    {
      heading: 'Type sau, sau ' + (index + 1),
      formInfo: {
        type: ALTERNATIVES,
        alternatives: sheepTypes,
        field: 'sheepType' + index
      }
    },
    {
      heading: 'Farge, sau ' + (index + 1),
      formInfo: {
        type: ALTERNATIVES,
        alternatives: sheepColors,
        field: 'injuryColor' + index
      }
    },
    {
      heading: 'Skade, sau ' + (index + 1),
      formInfo: {
        type: ALTERNATIVES,
        alternatives: injuryCategories,
        field: 'injuryCategory' + index
      },
      subcategories: {
        [injuryCategories[0]]: [
          {
            heading: 'Halter, sau ' + (index + 1),
            formInfo: {
              type: ALTERNATIVES,
              alternatives: limp,
              field: 'injury' + index
            }
          }
        ],
        [injuryCategories[1]]: [
          {
            heading: 'Hodeskade, sau ' + (index + 1),
            formInfo: {
              type: ALTERNATIVES,
              alternatives: headInjury,
              field: 'injury' + index
            }
          }
        ],
        [injuryCategories[2]]: [
          {
            heading: 'Kropsskade, sau ' + (index + 1),
            formInfo: {
              type: ALTERNATIVES,
              alternatives: bodyInjury,
              field: 'injury' + index
            }
          }
        ],
        [injuryCategories[3]]: [
          {
            heading: 'Blødningm sau ' + (index + 1),
            formInfo: {
              type: ALTERNATIVES,
              alternatives: bleeding,
              field: 'injury' + index
            }
          }
        ]
      }
    },
    {
      heading: 'Avgjørelse, sau ' + (index + 1),
      formInfo: {
        type: ALTERNATIVES,
        alternatives: decisions,
        field: 'decision' + index
      }
    }
  ];
}

const injuredStructure = [
  {
    heading: 'Skade',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: injuryCategories,
      field: 'injuryCategory'
    },
    subcategories: {
      [injuryCategories[0]]: [
        {
          heading: 'Halter',
          formInfo: {
            type: ALTERNATIVES,
            alternatives: limp,
            field: 'injury'
          }
        }
      ],
      [injuryCategories[1]]: [
        {
          heading: 'Hodeskade',
          formInfo: {
            type: ALTERNATIVES,
            alternatives: headInjury,
            field: 'injury'
          }
        }
      ],
      [injuryCategories[2]]: [
        {
          heading: 'Kropsskade',
          formInfo: {
            type: ALTERNATIVES,
            alternatives: bodyInjury,
            field: 'injury'
          }
        }
      ],
      [injuryCategories[3]]: [
        {
          heading: 'Blødning',
          formInfo: {
            type: ALTERNATIVES,
            alternatives: bleeding,
            field: 'injury'
          }
        }
      ]
    }
  },
  {
    heading: 'Avgjørelse',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: decisions,
      field: 'decision'
    }
  },
  {
    heading: 'Kontaktet',
    formInfo: {
      type: TEXT_FIELD,
      field: 'contacted'
    }
  }
];

const singleSheepStructure = [
  {
    heading: 'Farge',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: sheepColors,
      field: FIELD_COLOR
    }
  },
  {
    heading: 'Er denne sauen skadet?',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: yesNo,
      field: 'hasInjury'
    },
    subcategories: {
      [yesNo[0]]: injuredStructure
    }
  }
];

const groupStructure = [
  {
    heading: 'Antall søyer',
    formInfo: {
      type: SHEEP_COLOR,
      groupIndex: 0,
      alternatives: sheepColors,
      field: 'ewes'
    }
  },
  {
    heading: 'Antall lam',
    formInfo: {
      type: SHEEP_COLOR,
      groupIndex: 0,
      alternatives: sheepColors,
      field: 'lamb'
    }
  },
  {
    heading: 'Er noen av de registrerte sauene skadet?',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: yesNo,
      field: 'hasInjury'
    },
    subcategories: {
      [yesNo[0]]: [
        {
          heading: 'Hvor mange skadde sauer? ',
          formInfo: {
            type: COUNTER,
            field: 'count'
          }
        },
        {
          heading: 'Kontaktet',
          formInfo: {
            type: TEXT_FIELD,
            field: 'contacted'
          }
        }
      ]
    }
  }
];

export const sheepFormStructure = {
  hasCategories: false,
  elements: [{
    heading: 'Underkategori',
    formInfo: {
      type: ALTERNATIVES,
      alternatives: categories,
      field: 'category',
      icons: {
        [categories[0]]: `${baseUrl}ewe`,
        [categories[1]]: `${baseUrl}lamb`,
        [categories[2]]: `${baseUrl}sheep_with_lamb`
      }
    },
    subcategories: {
      [categories[0]]: singleSheepStructure,
      [categories[1]]: singleSheepStructure,
      [categories[2]]: groupStructure
    }
  }]
};

