import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as observationRoundActions from 'src/actions/observationRoundActions';
import * as routerActions from '../actions/routerActions';
import { openConfirmWindow } from '../utilities/warning_window.js';
import NavBar from './loadObservation/NavBar.jsx';
import { getFileMapperAsObject, deleteObservationRoundById } from '../utilities/save.js';
import { getDateTimeAsString } from '../utilities/dateParser.js';
import InfoPage from '../containers/loadObservation/InfoPage.jsx';
import * as transition from '../utilities/transitions.js';

class LoadObservationRound extends Component {

  constructor(props) {
    super(props);

    this.state = {
      files: getFileMapperAsObject(),
      openObservationRoundId: '',
      showInfoPage: false,
      headerText: 'Lagrede tilsynsturer',
      tab: 1
    };
    this.navigateBack = this.navigateBack.bind(this);
    this.openObservationRoundInfoPage = this.openObservationRoundInfoPage.bind(this);
    this.deleteObservationRound = this.deleteObservationRound.bind(this);
    this.loadRound = this.loadRound.bind(this);
    this.getActiveList = this.getActiveList.bind(this);
    this.getCompletedList = this.getCompletedList.bind(this);
    this.hideInfoPage = this.hideInfoPage.bind(this);
    this.onTabClick = this.onTabClick.bind(this);
  }

  componentWillMount() {
    document.addEventListener('backbutton', this.navigateBack, false);
  }

  componentDidMount() {
    transition.animateRightToLeft('load-observation-container');
  }

  componentWillUnmount() {
    document.removeEventListener('backbutton', this.navigateBack, false);
  }

  getActiveList() {
    let list = null;
    let listIsEmpty = true;
    if (this.state.files !== null && this.state.files !== undefined && Object.keys(this.state.files).length > 0) {
      list = Object.keys(this.state.files).map(id => {
        const file = this.state.files[id];
        const isOpenRound = id === this.props.observationRound.id;
        const date = getDateTimeAsString(file.createdTimestamp);
        if (!file.roundIsComplete) {
          listIsEmpty = false;
          return (
            <div className={`active-round-container${isOpenRound ? ' active' : ''}`} key={file.id} id={id}>
              <div className="active-round-header" onClick={() => { this.openObservationRoundInfoPage(id); }} >
                <i className={`active-round-icon fa ${isOpenRound ? 'fa-map' : 'fa-map-o'}`} />
                <div className="active-round-text">
                  {date}
                </div>
                <i className="fa active-round-angle fa-angle-right" />
              </div>
            </div>
          );
        }
      });
    }
    if (listIsEmpty) {
      return <div className="no-saved-rounds">Du har ingen aktive runder</div>;
    } else {
      return list;
    }
  }

  getCompletedList() {
    let list = null;
    let listIsEmpty = true;
    if (this.state.files !== null && this.state.files !== undefined && Object.keys(this.state.files).length > 0) {
      list = Object.keys(this.state.files).map(id => {
        const file = this.state.files[id];
        const date = getDateTimeAsString(file.createdTimestamp);
        if (file.roundIsComplete) {
          listIsEmpty = false;
          return (
            <div className="active-round-container" key={file.id} id={id}>
              <div className="active-round-header" onClick={() => { this.openObservationRoundInfoPage(id); }} >
                <i className="active-round-icon fa fa-map-o" />
                <div className="active-round-text">
                  {date}
                </div>
                <i className="fa active-round-angle fa-angle-right" />
              </div>
            </div>
          );
        }
      });
    }
    if (listIsEmpty) {
      return <div className="no-saved-rounds">Du har ingen fullførte runder</div>;
    } else {
      return list;
    }
  }

  onTabClick(tab) {
    this.setState({
      tab
    });
  }

  getHeaderWithText(text) {
    return (
      <div className="list-header">
        <i className="decorator-line" />
        <div className="list-header-text">
          {text}
        </div>
        <i className="decorator-line" />
      </div>
    );
  }

  openObservationRoundInfoPage(id) {
    this.setState({
      openObservationRoundId: id,
      showInfoPage: true,
      headerText: getDateTimeAsString(this.state.files[id].createdTimestamp)
    });
    transition.animateRightToLeft('observation-information');
  }

  loadRound(state) {
    this.props.loadObservationRound(state);
    transition.animateToLeftPosition('load-observation-container', '100vw', () => {
      this.props.routerActions.toggleSavedRoundsView();
    });
  }

  navigateBack() {
    if (this.state.showInfoPage) {
      transition.animateToLeftPosition('observation-information', '100vw', () => {
        this.hideInfoPage();
      });
      this.setState({
        files: getFileMapperAsObject()
      });
    } else {
      transition.animateToLeftPosition('load-observation-container', '100vw', () => {
        this.props.routerActions.toggleSavedRoundsView();
      });
    }
  }

  deleteObservationRound() {
    const isActiveRound = this.state.openObservationRoundId === this.props.observationRound.id;
    const extraText = isActiveRound ? ' PS: Dette er den pågående runden!' : '';
    openConfirmWindow('Er du sikker på at du ønsker å slette dataene fra denne tilsynsturen?' + extraText, () => {
      deleteObservationRoundById(this.state.openObservationRoundId);
      transition.animateToLeftPosition('observation-information', '100vw');
      const files = this.state.files;
      delete files[this.state.openObservationRoundId];
      transition.animateToLeftPosition('observation-information', '100vw', () => {
        this.setState({
          files,
          showInfoPage: false,
          openObservationId: '',
          headerText: 'Lagrede runder'
        });
      });
      if (isActiveRound) {
        this.props.onDeleteActiveRound();
      }
    });
  }

  hideInfoPage() {
    transition.animateToLeftPosition('observation-information', '100vw');
    this.setState({
      showInfoPage: false,
      headerText: 'Lagrede runder'
    });
  }

  getInfoPageView() {
    if (this.state.showInfoPage) {
      let state = this.props.observationRound;
      if (this.state.openObservationRoundId !== '') {
        state = this.state.files[this.state.openObservationRoundId];
      }
      return (
        <InfoPage
          state={state}
          onLoad={this.loadRound}
          roundHasBeenUploaded={this.props.observationRoundActions.roundHasBeenUploaded}
        />
      );
    } else {
      return null;
    }
  }

  render() {
    return (
      <div id="load-observation-container" className="load-observation-container">
        <NavBar
          label={this.state.headerText}
          onClick={this.navigateBack}
          delete={this.deleteObservationRound}
          showDeleteButton={this.state.showInfoPage}
        />
        {this.getInfoPageView()}
        <div className="active-rounds-container">
          <div className="tabs">
            <div className={`tab${this.state.tab === 1 ? ' active' : ''}`} onClick={() => this.onTabClick(1)}>
              Aktive
            </div>
            <div className={`tab${this.state.tab === 2 ? ' active' : ''}`} onClick={() => this.onTabClick(2)}>
              Fullførte
            </div>
          </div>
          <div className="active-bar-container">
            <div className={`active-bar ${'tab-' + this.state.tab}`} />
          </div>
          <div className={`tab-content${this.state.tab === 1 ? ' show' : ''}`}>
            {this.getActiveList()}
          </div>
          <div className={`tab-content${this.state.tab === 2 ? ' show' : ''}`}>
            {this.getCompletedList()}
          </div>
        </div>
      </div>);
  }
}

function mapStateToProps(state) {
  return {
    observationRound: state.observationRound,
    appRouter: state.router
  };
}

function mapDispatchToProps(dispatch) {
  return {
    observationRoundActions: bindActionCreators(observationRoundActions, dispatch),
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(LoadObservationRound);
