/* eslint-disable react/prop-types,brace-style,react/jsx-indent-props,react/jsx-first-prop-new-line,no-underscore-dangle,jsx-a11y/no-static-element-interactions,no-undef,class-methods-use-this,no-param-reassign */
import React, { Component } from 'react';
import 'MovingMarker';
import 'leaflet';
import 'leaflet.offline';
import 'leaflet-draw';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as mapActions from 'src/actions/mapActions';
import * as observationRoundActions from 'src/actions/observationRoundActions';
import * as routerActions from 'src/actions/routerActions';
import { hasConnection } from '../utilities/connectionDetection.js';
import { uploadAllRoundData } from '../utilities/apiCommunication.js';
import NewObservation from './NewObservation.jsx';
import SettingsView from './SettingsView.jsx';
import FinishRoundView from './FinishRound.jsx';
import NewObservationRound from './NewObservationRound.jsx';
import NavigationMenu from './mainMap/NavigationMenu.jsx';
import LoadObservationRound from './LoadObservationRound.jsx';
import NavBar from './mainMap/NavBar.jsx';
import DownloadNavBar from './mainMap/DownloadNavBar.jsx';
import ImageViewer from '../common/ImageViewer.jsx';
import { openConfirmWindow, openSuccessWindow } from '../utilities/warning_window';
import {
  saveFileMapperIsEmpty,
  createSaveFileMapper,
  getSettings,
  getSavedAreasAsList,
  saveArea,
  clearSavedAreas
} from '../utilities/save';
import {
  mapSettings,
  mapURL,
  userTrackerPolylineStyle,
  minTrackingDistance,
  savedAreaStyle
} from '../constants/map.js';
import {
  listHasElements,
  listIsEmpty,
  objectIsEmpty
} from '../utilities/commonHelpMethods';
import {
  getDistance,
  setMovingMarkerPosition,
  getObservationPosition,
  getDownloadController,
  checkForActiveRounds
} from '../utilities/mapHelper';
import AddButton from './mainMap/AddButton.jsx';
import {
  getMarkerIcon,
  observationPolyLineStyle,
  getObservationOptionsPopup
} from '../utilities/mapMarkers';

class Map extends Component {

  constructor(props) {
    super(props);

    this.state = {
      addButtonIsOpen: false,
      downloadViewIsActive: false,
      roundIsActive: false,
      hideReticle: true,
      observationImages: [],
      activeRegistrationPointId: '',
      activeObservationId: '',
      activePolylineId: '',
      hideImageViewer: true,
      progress: 0,
      currentlySavedTiles: 0,
      tilesToBeSaved: 0,
      totalStorage: 0,
      downloadState: 'not-downloading'
    };

    this.onLocationFound = this.onLocationFound.bind(this);
    this.onLocationError = this.onLocationError.bind(this);
    this.closeAddButton = this.closeAddButton.bind(this);
    this.toggleAddButton = this.toggleAddButton.bind(this);
    this.onNewObservationRoundSubmit = this.onNewObservationRoundSubmit.bind(this);
    this.onNewObservationClick = this.onNewObservationClick.bind(this);
    this.onRegistrationMarkerClick = this.onRegistrationMarkerClick.bind(this);
    this.onPopupClose = this.onPopupClose.bind(this);
    this.onPopupOpen = this.onPopupOpen.bind(this);
    this.onMoveStart = this.onMoveStart.bind(this);
    this.onDownloadMapMenuClick = this.onDownloadMapMenuClick.bind(this);
    this.exitDownloadView = this.exitDownloadView.bind(this);
    this.onStorageSize = this.onStorageSize.bind(this);
    this.onSaveStart = this.onSaveStart.bind(this);
    this.onSaveTileEnd = this.onSaveTileEnd.bind(this);
    this.onSaveEnd = this.onSaveEnd.bind(this);
    this.onTilesRemoved = this.onTilesRemoved.bind(this);
    this.loadObservationRound = this.loadObservationRound.bind(this);
    this.onDeleteActiveRound = this.onDeleteActiveRound.bind(this);
    this.onUserTrackingToggle = this.onUserTrackingToggle.bind(this);
    this.onFinishRound = this.onFinishRound.bind(this);
    this.navigateBack = this.navigateBack.bind(this);
    this.onNewObservationSubmit = this.onNewObservationSubmit.bind(this);
    this.onOnline = this.onOnline.bind(this);
    this.onAutomaticUploadToggle = this.onAutomaticUploadToggle.bind(this);
    this.closeImageViewer = this.closeImageViewer.bind(this);
    this.onDeleteTiles = this.onDeleteTiles.bind(this);
  }

  componentDidMount() {
    // Create local save file if it doesn't exist
    if (saveFileMapperIsEmpty()) {
      createSaveFileMapper();
    }

    // Change color of phone statusbar when app starts
    this.onDeviceReady = () => {
      StatusBar.backgroundColorByHexString('#3CA55C');
    };

    // Create map and go to where the user is
    this.map = L.map('main-map').locate({ setView: true, maxZoom: 16 });

    // Get map data from url and define map settings
    const baseLayer = L.tileLayer.offline(mapURL, mapSettings).addTo(this.map);

    // Set map listeners
    this.setListeners(baseLayer);

    // Add controls for downloading tiles for offline use
    this.control = L.control.savetiles(baseLayer, getDownloadController(this.onDeleteTiles)).addTo(this.map);

    this.downloadRect = new L.Rectangle([[0, 0], [0, 0]]);

    // Define and add controls for drawable items. Used mainly for the download tiles part.
    this.drawnItems = new L.FeatureGroup();
    this.map.addLayer(this.drawnItems);
    this.drawControl = new L.Control.Draw({
      edit: {
        featureGroup: this.drawnItems
      }
    });
    this.map.addControl(this.drawControl);

    // Initialize user marker
    this.movingMarker = setMovingMarkerPosition(this.map);

    this.userTrackingPoints = this.props.observationRound.userTrackingPoints;
    this.userTrackingPolyline = L.polyline([this.userTrackingPoints], userTrackerPolylineStyle);

    // Initialize featureGroups to hold different layers
    this.initLayerGroups(this);

    // Give user the choice to open saved rounds view if active rounds exists
    checkForActiveRounds(this.props.routerActions.toggleSavedRoundsView);

    // Watch user position and move based on retrieved coordinates
    this.watchPosition();

    document.addEventListener('backbutton', this.navigateBack, false);
    document.addEventListener('deviceready', this.onDeviceReady, false);
  }

  componentWillUnmount() {
    document.removeEventListener('backbutton', this.navigateBack, false);
    document.removeEventListener('online', this.onOnline, false);
    document.removeEventListener('deviceready', this.onDeviceReady, false);
  }

  onOnline() {
    uploadAllRoundData();
  }

  onDeleteActiveRound() {
    this.clearAllLayers();
    this.setState({
      roundIsActive: false
    });
  }

  // Save new position if user has walked a certain distance since last saved point
  onLocationFound(latitude, longitude) {
    const trackingPoints = this.props.observationRound.userTrackingPoints;
    let previousTrackingPoint;
    let distanceToPreviousPoint;
    if (trackingPoints.length > 0) {
      previousTrackingPoint = trackingPoints[trackingPoints.length - 1];
      distanceToPreviousPoint = getDistance(latitude, longitude, previousTrackingPoint[0], previousTrackingPoint[1]);
    }
    const newTrackingPointShouldBeAdded = listIsEmpty(trackingPoints) || (listHasElements(trackingPoints) && distanceToPreviousPoint > minTrackingDistance);
    if (newTrackingPointShouldBeAdded) {
      this.movingMarker.moveTo(new L.LatLng(latitude, longitude), 1000);
      this.movingMarker.start();
      this.props.mapActions.updateUserPosition(latitude, longitude);
      if (this.state.roundIsActive) {
        this.props.observationRoundActions.addUserTrackingPoint([latitude, longitude]);
        this.userTrackingPolyline.addLatLng([latitude, longitude]);
      }
    }
  }

  onLocationError(e) {
    console.log('location error' + e);
  }

  // Hide add button if user moves the map
  onMoveStart() {
    if (this.state.addButtonIsOpen) {
      this.setState({
        addButtonIsOpen: false
      });
    }
  }

  // Create new marker when submitting new observation, then open the popup
  onNewObservationSubmit(observationId, observationData) {
    this.map.closePopup();
    this.setState({
      observationImages: observationData.images
    });
    const observationMarker = this.observationMarkerGroup.getLayer(observationId);
    observationMarker.setIcon(getMarkerIcon(observationData));
    observationMarker.bindPopup(getObservationOptionsPopup(observationId, observationData));
    observationMarker.openPopup();
  }

  // Clear map graphics and set round to active once a new observation round is started.
  onNewObservationRoundSubmit() {
    this.clearAllLayers();
    this.setState({
      roundIsActive: true,
      hideReticle: false
    });
    const currentPosition = this.props.map.currentUserPosition;
    this.userTrackingPolyline = L.polyline([[currentPosition.latitude, currentPosition.longitude]], userTrackerPolylineStyle);
    this.userTrackingPolyline.addTo(this.map);
    this.onUserTrackingToggle(getSettings().userTrackingIsOn);
    this.props.observationRoundActions.addUserTrackingPoint([currentPosition.latitude, currentPosition.longitude]);
    this.drawHomeMarker(currentPosition);
  }

  // Show/hide observations related to a registration points when it is clicked
  onRegistrationMarkerClick(e) {
    this.toggleChildrenVisibilityByRegistrationPointId(e.target._leaflet_id);
  }

  onPopupClose() {
    this.setState({
      hideReticle: false
    });
  }

  // Show statistics about the observation, and add click event listeners to the buttons when opening a popup
  onPopupOpen(registrationPointId, observationId, polylineId, isLoadingData, observationData) {
    let images = observationData.images;
    if (!isLoadingData) {
      const regPoint = this.props.observationRound.registrationPoints[registrationPointId];
      console.log(regPoint);
      if (observationId in regPoint.observations) {
        images = regPoint.observations[observationId].data.images;
      }
    }
    this.setState({
      hideReticle: true,
      activeObservationId: observationId,
      activeRegistrationPointId: registrationPointId,
      activePolylineId: polylineId,
      observationImages: images
    });
    const markerOptions = document.getElementsByClassName('option-container');
    Array.from(markerOptions).forEach(element => {
      element.addEventListener('click', () => this.handleMarkerOptionsByType(element, observationId, registrationPointId, polylineId));
    });
  }

  // Prevent the user from being able to open popup of hidden markers
  onMarkerClick(e, registrationPointId) {
    if (!this.props.observationRound.registrationPoints[registrationPointId].childrenIsVisible) {
      e.target.closePopup();
    }
  }

  // If user is not close to an exisiting registrationpoint, spawn a new one. If not, show all observations related to
  // the nearest registrationPoint. Spawn a new observation marker and open it's popup.
  onNewObservationClick() {
    const userPosition = this.props.map.currentUserPosition;
    const observationCoords = getObservationPosition(this.map);
    let registrationPoint = this.getRegistrationPoint(this.props.observationRound.registrationPoints, userPosition);
    if (registrationPoint.id === -1) {
      registrationPoint = this.drawRegistrationMarker(userPosition);
      this.props.observationRoundActions.addNewRegistrationPoint(registrationPoint.id, registrationPoint.position);
    } else if (!this.props.observationRound.registrationPoints[registrationPoint.id].childrenIsVisible) {
      this.toggleChildrenVisibilityByRegistrationPointId(registrationPoint.id);
    }
    const polylineId = this.drawObservationPolyline(registrationPoint.position, observationCoords, registrationPoint.id);
    const observationId = this.drawObservationMarker(observationCoords, registrationPoint.id, polylineId);

    this.props.observationRoundActions.addNewObservation(registrationPoint.id, observationId, observationCoords);
    this.closeAddButton();
  }

  // Open "download view". Hide the normal map graphics and spawn the square and buttons used for downloading map data.
  onDownloadMapMenuClick() {
    this.map.removeLayer(this.downloadRect);
    this.setState({
      downloadViewIsActive: true,
      hideReticle: true,
      progress: 0,
      currentlySavedTiles: 0,
      tilesToBeSaved: 0,
      downloadState: 'not-downloading'
    });
    this.setMapGraphicsOpacity(
      this.registrationMarkerGroup, this.homeMarkerGroup,
      this.observationMarkerGroup, this.polylineGroup,
      this.userTrackingPolyline, this.map, 0
    );
    this.downloadRect = new L.Rectangle(this.map.getBounds(), { id: 'downloadRect' });
    this.spawnEditableRectangle(this.downloadRect, this.map, this.drawnItems);
    const savedAreas = getSavedAreasAsList();
    for (let i = 0; i < savedAreas.length; i += 1) {
      const savedArea = savedAreas[i];
      new L.Rectangle([[savedArea._northEast.lat, savedArea._northEast.lng], [savedArea._southWest.lat, savedArea._southWest.lng]], savedAreaStyle).addTo(this.savedAreas);
    }
  }

  // Trigger download click
  onDownloadMapClick() {
    const buttons = document.querySelectorAll(('.leaflet-draw-actions a'));
    buttons[0].click();
    document.querySelector('.download-button').click();
  }

  // Trigger delete tiles click
  onDeleteTiles() {
    if (this.state.downloadViewIsActive) {
      this.map.removeLayer(this.downloadRect);
      this.downloadRect = new L.Rectangle(this.map.getBounds(), { id: 'downloadRect' });
      this.spawnEditableRectangle(this.downloadRect, this.map, this.drawnItems);
      this.setState({
        downloadState: 'not downloading'
      });
    }
  }

  // Update total amount of data saved once downloading is complete.
  onStorageSize(e) {
    this.setState({
      totalStorage: e.storagesize
    });
  }

  // Reset download progress and change style of the rectangle representing the area being downloaded
  onSaveStart(e) {
    this.setState({
      progress: 0,
      currentlySavedTiles: 0,
      tilesToBeSaved: e._tilesforSave.length,
      downloadState: 'downloading'
    });
    this.downloadRect.setStyle({
      fillColor: '#3388ff',
      color: '#3388ff'
    });
  }

  // For each tile that gets saved, update progressbar and progress number
  onSaveTileEnd() {
    const progress = ((this.state.currentlySavedTiles + 1) / this.state.tilesToBeSaved) * 100;
    this.setState({
      currentlySavedTiles: this.state.currentlySavedTiles + 1,
      progress
    });
  }

  // Give feedback to user once download is done, spawn a new rectangle indicating that the area is saved.
  onSaveEnd() {
    openSuccessWindow('Nedlasting fullført!');
    this.setState({
      downloadState: 'done'
    });
    const savedArea = new L.Rectangle(this.downloadRect.getBounds(), savedAreaStyle);
    saveArea(this.downloadRect.getBounds());
    savedArea.addTo(this.savedAreas);
    this.map.removeLayer(this.downloadRect);
  }

  // Triggers after all data is deleted. Reset progressbar, amount of saved tiles etc. Clear graphics.
  onTilesRemoved() {
    this.setState({
      progress: 0,
      currentlySavedTiles: 0,
      tilesToBeSaved: 0,
      totalStorage: 0
    });
    clearSavedAreas();
    this.savedAreas.clearLayers();
  }

  // Turn on/off automatic upload. If on, add event listener for when connecting to internet and try to upload data.
  // If off, remove event listener for when connecting to internet.
  onAutomaticUploadToggle(automaticUploadIsOn) {
    if (automaticUploadIsOn) {
      document.addEventListener('online', this.onOnline);
      if (hasConnection()) {
        this.onOnline();
      }
    } else {
      document.removeEventListener('online', this.onOnline);
    }
  }

  // Toggle opacity of user tracking line
  onUserTrackingToggle(userTrackingIsOn) {
    switch (userTrackingIsOn) {
      case true: {
        this.userTrackingPolyline.setStyle({
          opacity: 1
        });
        break;
      }

      case false: {
        this.userTrackingPolyline.setStyle({
          opacity: 0
        });
        break;
      }

      default: {
        console.log('no value');
      }
    }
  }

  onFinishRound() {
    this.setState({
      roundIsActive: false
    });
  }

  /** ************* Methods for view rendering, made to make render function easier to read ************** */

  getAddButton() {
    if (this.state.roundIsActive) {
      return (
        <AddButton
          show={this.state.addButtonIsOpen}
          roundIsActive={this.state.roundIsActive}
          onNewObservationClick={this.onNewObservationClick}
          totalStorage={this.state.totalStorage}
          onNewObservationRoundClick={() => this.openView('NewObservationRound')}
          onAddButtonClick={this.toggleAddButton}
          hideAddButton={this.state.downloadViewIsActive}
        />
      );
    } else {
      return null;
    }
  }

  getNavBar() {
    if (this.state.downloadViewIsActive) {
      return (
        <DownloadNavBar
          closeDownloadView={this.exitDownloadView}
          totalStorage={this.state.totalStorage}
          progress={this.state.progress}
          currentlySavedTiles={this.state.currentlySavedTiles}
          tilesToBeSaved={this.state.tilesToBeSaved}
        />
      );
    } else {
      return <NavBar />;
    }
  }

  getSettingsView(router) {
    if (router.showSettingsView) {
      return (
        <SettingsView
          onUserTrackingToggle={this.onUserTrackingToggle}
          onAutomaticUploadToggle={this.onAutomaticUploadToggle}
          onDeleteTiles={this.triggerDeleteTilesClick}
          totalStorage={this.state.totalStorage}
        />
      );
    } else {
      return null;
    }
  }

  getNewObservationView(router) {
    if (router.showNewObservationView) {
      return (
        <NewObservation
          registrationPointId={this.state.activeRegistrationPointId}
          observationId={this.state.activeObservationId}
          onSubmit={this.onNewObservationSubmit}
        />
      );
    } else {
      return null;
    }
  }

  getNewObservationRoundView(router) {
    if (router.showNewObservationRoundView) {
      return (
        <NewObservationRound
          roundIsActive={this.state.roundIsActive}
          onSubmit={this.onNewObservationRoundSubmit}
        />
      );
    } else {
      return null;
    }
  }

  getNavigationMenuView(router) {
    if (router.showNavigationMenuView) {
      return (
        <NavigationMenu
          roundIsActive={this.state.roundIsActive}
          onDownloadMapClick={this.onDownloadMapMenuClick}
        />
      );
    } else {
      return null;
    }
  }

  getFinishRoundView(router) {
    if (router.showFinishRoundView) {
      return (
        <FinishRoundView
          roundIsActive={this.state.roundIsActive}
          onFinishRound={this.onFinishRound}
        />
      );
    } else {
      return null;
    }
  }

  getSavedRoundsView(router) {
    if (router.showSavedRoundsView) {
      return (
        <LoadObservationRound
          onDeleteActiveRound={this.onDeleteActiveRound}
          loadObservationRound={this.loadObservationRound}
        />
      );
    } else {
      return null;
    }
  }

  getDownloadButton() {
    switch (this.state.downloadState) {
      case 'downloading': {
        return (
          <div className="download-map-button" >
            <i className="fa fa-spinner fa-pulse fa-fw" /> Laster ned
          </div>
        );
      }
      case 'done': {
        return (
          <div className="download-map-button" onClick={this.onDownloadMapMenuClick}>
            Ny nedlasting
          </div>
        );
      }
      default: {
        return (
          <div className="download-map-button" onClick={this.onDownloadMapClick} >
            Last ned
          </div>
        );
      }
    }
  }

  getImageViewer() {
    const openObservationIds = {
      registrationPointId: this.state.activeRegistrationPointId,
      observationId: this.state.activeObservationId
    };
    return (
      <ImageViewer
        images={this.state.observationImages}
        close={this.closeImageViewer}
        hide={this.state.hideImageViewer}
        onDeleteImage={this.props.observationRoundActions.deleteImage}
        openObservationIds={openObservationIds}
      />
    );
  }

  /** ************************************************************************ * */

  setListeners(layer) {
    layer.on('storagesize', this.onStorageSize);
    layer.on('savestart', this.onSaveStart);
    layer.on('savetileend', this.onSaveTileEnd);
    layer.on('saveend', this.onSaveEnd);
    layer.on('tilesremoved', this.onTilesRemoved);
    this.map.on('locationfound', e => { this.onLocationFound(e.latitude, e.longitude); });
    this.map.on('locationerror', this.onLocationError);
    this.map.on('movestart', this.onMoveStart);
    //this.map.on('click', e => { this.onLocationFound(e.latlng.lat, e.latlng.lng); });
  }

  // Get the closes registrationPoint that is within a certain diameter
  getRegistrationPoint(registrationPoints, userPosition) {
    let closestPointId = -1;
    let closestPointDistance = 10000;
    let closestPointCoords = {};

    Object.keys(registrationPoints).forEach(id => {
      const point = registrationPoints[id];
      const pointDistance = getDistance(point.latitude, point.longitude, userPosition.latitude, userPosition.longitude);
      if (pointDistance <= 0.01 && pointDistance <= closestPointDistance) {
        closestPointDistance = pointDistance;
        closestPointId = parseInt(id, 10);
        closestPointCoords = {
          latitude: point.latitude,
          longitude: point.longitude
        };
      }
    });

    return {
      id: closestPointId,
      position: closestPointCoords
    };
  }

  // Sets opacity of all graphics
  setMapGraphicsOpacity(registrationMarkers, homeMarkers, observationMarkers, markerPolyline, userPolyline, map, opacity) {
    map.closePopup();
    registrationMarkers.eachLayer(layer => {
      layer.setOpacity(opacity);
    });
    homeMarkers.eachLayer(layer => {
      layer.setOpacity(opacity);
    });
    observationMarkers.eachLayer(layer => {
      layer.setOpacity(opacity);
    });
    markerPolyline.eachLayer(layer => {
      layer.setStyle({
        opacity
      });
    });
    userPolyline.setStyle({
      opacity
    });
  }

  spawnEditableRectangle(rect, map, layerGroup) {
    map.zoomOut(1);
    rect.addTo(layerGroup);
    document.querySelector('.leaflet-draw-edit-edit').click();
  }

  // Close download "view". Meaning turn off all graphics related to downloading and turn back on all other graphics.
  exitDownloadView() {
    this.setState({
      downloadViewIsActive: false,
      hideReticle: !this.state.roundIsActive
    });
    const userPosition = this.props.map.currentUserPosition;
    this.map.removeLayer(this.downloadRect);
    this.savedAreas.clearLayers();
    this.map.flyTo([userPosition.latitude, userPosition.longitude]);
    this.setMapGraphicsOpacity(
      this.registrationMarkerGroup, this.homeMarkerGroup,
      this.observationMarkerGroup, this.polylineGroup,
      this.userTrackingPolyline, this.map, 1
    );
  }

  triggerDeleteTilesClick() {
    document.querySelector('.rmtiles').click();
  }

  drawHomeMarker(position = this.props.observationRound.homePosition) {
    if (!objectIsEmpty(position)) {
      this.props.observationRoundActions.registerHomePosition(position);
      this.homeMarkerGroup.clearLayers();
      const homeMarker = L.marker([position.latitude, position.longitude], { icon: getMarkerIcon({ type: 'home' }) });
      homeMarker.addTo(this.homeMarkerGroup);
    }
  }

  // Creates a new observation marker. Save it's parent on the marker object and return the id generated by leaflet when creating the marker.
  drawObservationMarker(observationCoords, registrationPointId, polylineId, isLoadingData = false, observationData = {}) {
    const observationPosition = [observationCoords.latitude, observationCoords.longitude];
    const observationMarker = L.marker(
      observationPosition,
      {
        icon: getMarkerIcon(observationData),
        registrationPointId
      });
    observationMarker.addTo(this.observationMarkerGroup);
    const observationId = observationMarker._leaflet_id;
    const popup = observationMarker.bindPopup(getObservationOptionsPopup(observationId, observationData));
    observationMarker.on('click', e => this.onMarkerClick(e, registrationPointId));
    popup.on('popupclose', this.onPopupClose);
    popup.on('popupopen', () => this.onPopupOpen(registrationPointId, observationId, polylineId, isLoadingData, observationData));
    observationMarker.openPopup();
    return observationId;
  }

  // Creates a new polyine between a new observation and a registration marker.
  // Save it's parent on the marker object and return the id generated by leaflet when creating the marker.
  drawObservationPolyline(registrationCoords, observationCoords, registrationPointId) {
    const registrationPosition = [registrationCoords.latitude, registrationCoords.longitude];
    const observationPosition = [observationCoords.latitude, observationCoords.longitude];
    const polyline = L.polyline(
      [registrationPosition, observationPosition],
      {
        ...observationPolyLineStyle,
        registrationPointId
      });
    polyline.addTo(this.polylineGroup);
    return polyline._leaflet_id;
  }

  // Creates a new registration marker and returns an object containing its id and position
  drawRegistrationMarker(registrationCoords) {
    const registrationPosition = [registrationCoords.latitude, registrationCoords.longitude];

    const registrationMarker = L.marker(
      registrationPosition,
      {
        icon: getMarkerIcon({ type: 'registration' })
      }).on('click', this.onRegistrationMarkerClick);
    registrationMarker.addTo(this.registrationMarkerGroup);

    return {
      id: registrationMarker._leaflet_id,
      position: registrationCoords
    };
  }

  // Handles all button clicks in the popup for observation markers.
  handleMarkerOptionsByType(element, observationId, registrationPointId, polylineId) {
    const type = element.id.split('x')[0];
    switch (type) {
      case 'open': {
        this.openView('NewObservation');
        break;
      }
      case 'delete': {
        openConfirmWindow(
          'Er du sikker på at du ønsker å slette observasjonen?',
          () => {
            this.map.closePopup();
            this.observationMarkerGroup.removeLayer(observationId);
            this.polylineGroup.removeLayer(polylineId);
            if (Object.keys(this.props.observationRound.registrationPoints[registrationPointId].observations).length === 1) {
              this.registrationMarkerGroup.removeLayer(registrationPointId);
            }
            this.props.observationRoundActions.deleteObservation(observationId, registrationPointId);
          });
        break;
      }
      case 'close-popup': {
        this.map.closePopup();
        break;
      }
      case 'images': {
        this.setState({
          hideImageViewer: false
        });
        break;
      }
      default:
        break;
    }
  }

  // Turn on/off the observation related to a registrationPoint with the given id
  toggleChildrenVisibilityByRegistrationPointId(registrationPointId) {
    const registrationPoint = this.props.observationRound.registrationPoints[registrationPointId];
    if (registrationPoint.childrenIsVisible) {
      this.hideObservationMarkerAndPolyline(registrationPointId);
    }
    else {
      this.observationMarkerGroup.eachLayer(observation => {
        if (observation.options.registrationPointId === registrationPointId) {
          observation.setOpacity(1);
        }
      });
      this.polylineGroup.eachLayer(line => {
        if (line.options.registrationPointId === registrationPointId) {
          line.setStyle({
            opacity: 1
          });
        }
      });
    }
    this.props.observationRoundActions.toggleChildrenVisibility(registrationPointId);
  }

  // Hide observation marker and its related polyline based on registrationPoint id
  hideObservationMarkerAndPolyline(id) {
    this.observationMarkerGroup.eachLayer(observation => {
      if (observation.options.registrationPointId === id) {
        observation.setOpacity(0);
      }
    });
    this.polylineGroup.eachLayer(line => {
      if (line.options.registrationPointId === id) {
        line.setStyle({
          opacity: 0
        });
      }
    });
  }

  // Show/hide button for adding a new observation
  toggleAddButton() {
    console.log(this.props.observationRound);
    this.map.closePopup();
    let hideReticle = true;
    if (this.state.roundIsActive) {
      hideReticle = false;
    }
    this.setState({
      addButtonIsOpen: !this.state.addButtonIsOpen,
      hideReticle
    });
  }

  closeAddButton() {
    this.setState({
      addButtonIsOpen: false
    });
  }

  openView(view) {
    switch (view) {
      case 'NewObservationRound': {
        this.props.routerActions.toggleNewObservationRoundView();
        break;
      }
      case 'NewObservation': {
        this.props.routerActions.toggleNewObservationView();
        break;
      }
      default:
        break;
    }
    this.closeAddButton();
  }

  // Remove all drawn graphics from map
  clearAllLayers() {
    this.map.removeLayer(this.userTrackingPolyline);
    this.homeMarkerGroup.clearLayers();
    this.registrationMarkerGroup.clearLayers();
    this.observationMarkerGroup.clearLayers();
    this.polylineGroup.clearLayers();
    this.savedAreas.clearLayers();
  }


  // Create new layergroups and add them to the map
  initLayerGroups(that) {
    that.homeMarkerGroup = new L.featureGroup();
    that.registrationMarkerGroup = new L.featureGroup();
    that.observationMarkerGroup = new L.featureGroup();
    that.polylineGroup = new L.featureGroup();
    that.savedAreas = new L.featureGroup();
    that.homeMarkerGroup.addTo(that.map);
    that.registrationMarkerGroup.addTo(that.map);
    that.observationMarkerGroup.addTo(that.map);
    that.polylineGroup.addTo(that.map);
    that.savedAreas.addTo(that.map);
    that.userTrackingPolyline.addTo(that.map);
  }

  closeImageViewer() {
    this.setState({
      hideImageViewer: true
    });
    this.map.closePopup();
    const observationMarker = this.observationMarkerGroup.getLayer(this.state.activeObservationId);
    const data = this.props.observationRound.registrationPoints[this.state.activeRegistrationPointId].observations[this.state.activeObservationId].data;
    observationMarker.setIcon(getMarkerIcon(data));
    observationMarker.bindPopup(getObservationOptionsPopup(this.state.activeObservationId, data));
    observationMarker.openPopup();
  }

  // Remove all current map graphics. Then iterate over the observation data and draw all graphics on the map
  // related to the observation data. Used for opening saved rounds form memory.
  loadObservationRound(observationData) {
    this.clearAllLayers();
    const observationRound = observationData;
    const registrationPoints = observationRound.registrationPoints;
    this.userTrackingPoints = observationRound.userTrackingPoints;

    this.drawHomeMarker(observationRound.homePosition);
    const newRegistrationPoints = {};
    Object.keys(registrationPoints).forEach(registrationKey => {
      const registrationPoint = registrationPoints[registrationKey];
      const observations = registrationPoint.observations;
      const newObservations = {};
      const registrationPosition = {
        latitude: registrationPoint.latitude,
        longitude: registrationPoint.longitude
      };
      const registrationPointData = this.drawRegistrationMarker(registrationPosition);
      Object.keys(observations).forEach(observationKey => {
        const observation = observations[observationKey];
        const observationPosition = {
          latitude: observation.latitude,
          longitude: observation.longitude
        };
        const polylineId = this.drawObservationPolyline(registrationPosition, observationPosition, registrationPointData.id);
        const observationId = this.drawObservationMarker(observationPosition, registrationPointData.id, polylineId, true, observation.data);
        newObservations[observationId] = observation;
      });
      registrationPoint.observations = newObservations;
      newRegistrationPoints[registrationPointData.id] = registrationPoint;
    });
    observationRound.registrationPoints = newRegistrationPoints;
    this.props.observationRoundActions.updateState(observationRound);
    this.map.closePopup();
    this.setState({
      roundIsActive: true
    });
    this.userTrackingPolyline = L.polyline([this.userTrackingPoints], userTrackerPolylineStyle);
    this.userTrackingPolyline.addTo(this.map);
  }

  // Method that checks if user has moved and updates the user position on the map
  watchPosition() {
    navigator.geolocation.watchPosition(
      e => { console.log('geowatch'); this.onLocationFound(e.coords.latitude, e.coords.longitude); },
      this.onLocationError,
      { maximumAge: 10000, timeout: 20000, enableHighAccuracy: true });

    if (getSettings().automaticUploadIsEnabled) {
      document.addEventListener('online', this.onOnline, false);
      this.onOnline();
      if (hasConnection()) {
        this.onOnline();
      }
    }
  }

  navigateBack() {
    if (this.state.downloadState !== 'not-downloading') {
      this.exitDownloadView();
    }
  }

  render() {
    const appRouter = this.props.appRouter;
    return (
      <div>
        {this.getImageViewer()}
        {this.getNewObservationRoundView(appRouter)}
        {this.getNewObservationView(appRouter)}
        {this.getNavigationMenuView(appRouter)}
        {this.getSavedRoundsView(appRouter)}
        {this.getSettingsView(appRouter)}
        {this.getFinishRoundView(appRouter)}
        {this.getNavBar()}
        <div id="main-map" />
        <img src="img/map-reticle.png" alt="Map reticle" className={`reticle${this.state.hideReticle ? ' hide' : ''}`} />
        {this.getAddButton()}
        <div className={`download-buttons${this.state.downloadViewIsActive ? ' show' : ''}`}>
          {this.getDownloadButton()}
          <div className={`delete-files-button${this.state.totalStorage > 0 && this.state.downloadState !== 'downloading' ? ' show' : ''}`} onClick={this.triggerDeleteTilesClick}>
            <i className="fa fa-trash" />
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    appRouter: state.router,
    map: state.map,
    observationRound: state.observationRound
  };
}

function mapDispatchToProps(dispatch) {
  return {
    mapActions: bindActionCreators(mapActions, dispatch),
    observationRoundActions: bindActionCreators(observationRoundActions, dispatch),
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(Map);
