/* eslint-disable react/prop-types */
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as transition from '../utilities/transitions.js';
import * as mapActions from '../actions/mapActions';
import * as routerActions from '../actions/routerActions';
import * as observationFormActions from '../actions/observationFormActions';
import * as ObservationRoundActions from '../actions/observationRoundActions.js';
import CollapsiblePanel from './newObservation/CollapsiblePanel.jsx';
import Alternatives from '../containers/newObservation/panelBodyElement/Alternatives.jsx';
import Counter from './newObservation/panelBodyElement/Counter.jsx';
import Images from '../containers/newObservation/panelBodyElement/Images.jsx';
import TextField from '../containers/newObservation/panelBodyElement/TextField.jsx';
import CategorySelection from './newObservation/CategorySelection.jsx';
import * as observationCategory from '../constants/observation/NewObservationCategories';
import SheepColor from './newObservation/panelBodyElement/SheepColor.jsx';
import { getUpdateDataFieldFunction, getHeading, getNumberOfPanels, getCategory, getKeysToDelete, arrayContains, getFormattedObservationData } from '../utilities/newObservationHelper';
import NavigationBar from './newObservation/NavigationBar.jsx';
import NextButton from './newObservation/NextButton.jsx';
import { deadSheepFormStructure } from '../constants/observation/deadSheep';
import { dogFormStructure } from '../constants/observation/dog';
import { injuredSheepFormStructure } from '../constants/observation/injuredSheep';
import { predatorFormStructure } from '../constants/observation/predator';
import { reindeerFormStructure } from '../constants/observation/reindeer';
import { SHEEP_GROUP, sheepFormStructure, getInjuryStructure, categories } from '../constants/observation/sheep';
import { woolFormStructure } from '../constants/observation/wool';
import { ALTERNATIVES, COUNTER, SHEEP_COLOR, TEXT_FIELD } from '../constants/observation/formElementtTypes';
import { MAIN_MAP } from '../constants/paths';

class NewObservation extends Component {
  constructor(props) {
    super(props);

    this.isSelectedPanel = this.isSelectedPanel.bind(this);
    this.onPanelClick = this.onPanelClick.bind(this);
    this.changeSubCategory = this.changeSubCategory.bind(this);
    this.changeSubCategory = this.changeSubCategory.bind(this);
    this.goToNextPanel = this.goToNextPanel.bind(this);
    this.handleChangeCategory = this.handleChangeCategory.bind(this);
    this.willDisplayPanel = this.willDisplayPanel.bind(this);
    this.getCollapsiblePanelElement = this.getCollapsiblePanelElement.bind(this);
    this.saveObservation = this.saveObservation.bind(this);
    this.navigateBack = this.navigateBack.bind(this);
    this.saveObservation = this.saveObservation.bind(this);
    this.getFlatFormElementStructure = this.getFlatFormElementStructure.bind(this);
    this.getFlatFormElementList = this.getFlatFormElementList.bind(this);

    this.firstTimeOpeningCategoryPanel = true;
  }

  componentWillMount() {
    const { observationId, registrationPointId } = this.props;
    const { registrationPoints } = this.props.observationRound;
    let observationData = { type: '' };
    try {
      observationData = registrationPoints[registrationPointId].observations[observationId].data;
    } catch (e) {
      console.error(e);
    }

    const isEdit = Object.keys(observationData).length > 3 || observationData.type.length > 0;

    if (isEdit) {
      const state = { ...this.props.observationForm };
      state.data = observationData;
      const type = observationData.type.length > 0 ? observationData.type : observationData.category;
      state.category = getCategory(type);
      state.index = -1;
      state.displayIndex = getNumberOfPanels(state) + 1;
      state.edit = true;

      this.props.observationFormActions.setFormState(state);
    } else {
      this.props.observationFormActions.setIndex(0);
    }
  }

  componentDidMount() {
    transition.animateRightToLeft('new-observation');
    document.addEventListener('backbutton', this.navigateBack, false);
  }

  componentWillUnmount() {
    document.removeEventListener('backbutton', this.navigateBack, false);
  }

  isEdit() {
    const registrationPointId = this.props.registrationPointId;
    const observationId = this.props.observationId;
    const { registrationPoints } = this.props.observationRound;
    let observationData = { type: '' };
    try {
      observationData = registrationPoints[registrationPointId].observations[observationId].data;
    } catch (e) {
      console.error(e);
    }

    return Object.keys(observationData).length > 3 || observationData.type.length > 0;
  }

  onPanelClick(index) {
    let i = index;
    if (this.isSelectedPanel(i)) {
      i = -1;
    }
    this.props.observationFormActions.setCurrentPanel(i);
  }

  getSheepAlternativeValue(formInfo, formData) {
    if (formInfo.field === 'color' && (formData.category === categories[0] || formData.category === categories[1])) {
      let colorObject = {};
      if (formData.category === categories[0]) {
        colorObject = formData.groups[0].ewes;
      } else if (formData.category === categories[1]) {
        colorObject = formData.groups[0].lamb;
      }

      if (colorObject.black > 0) {
        return 'Svart';
      } else if (colorObject.white > 0) {
        return 'Hvit';
      } else if (colorObject.blackAndWhite > 0) {
        return 'Svart og hvit';
      }
    }
    return null;
  }

  getBody(formInfo, isCategory = false) {
    const formActions = this.props.observationFormActions;
    const formData = this.props.observationForm.data;
    const updateDataField = getUpdateDataFieldFunction(formData.category, formInfo, formActions, this.handleChangeCategory);

    const sheepValue = this.getSheepAlternativeValue(formInfo, formData);

    const goToNextPanel = () => {
      this.goToNextPanel(formInfo.field);
    };

    switch (formInfo.type) {
      case ALTERNATIVES:
        return (
          <div>
            <Alternatives
              alternatives={formInfo.alternatives}
              field={formInfo.field}
              updateDataField={updateDataField}
              onSubmit={isCategory ? this.changeSubCategory : goToNextPanel}
              value={sheepValue !== null ? sheepValue : formData[formInfo.field]}
              icons={formInfo.icons !== undefined ? formInfo.icons : {}}
            />
          </div>
        );
      case TEXT_FIELD:
        return (
          <div>
            <TextField
              field={formInfo.field}
              updateDataField={formActions.updateDataField}
              value={formData[formInfo.field]}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      case SHEEP_COLOR: {
        return (
          <div>
            <SheepColor
              field={formInfo.field}
              groupIndex={formInfo.groupIndex}
              updateDataField={updateDataField}
              value={formData.groups[formInfo.groupIndex][formInfo.field]}
              key={formInfo.groupIndex}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      }
      case COUNTER: {
        return (
          <div>
            <Counter
              field={formInfo.field}
              updateDataField={formActions.updateDataField}
              formActions={formActions}
              value={formData[formInfo.field]}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      }
      default:
        return null;
    }
  }

  getNextButton() {
    return (
      <NextButton onClick={this.goToNextPanel} />
    );
  }

  getStandardFormElements() {
    const formActions = this.props.observationFormActions;
    const { observationForm } = this.props;
    const nextButton = this.getNextButton();

    return [
      {
        heading: 'Bilder',
        body: (
          <div>
            <Images observationFormActions={this.props.observationFormActions} observationForm={this.props.observationForm} />
            {this.showNextButton() ? nextButton : null}
          </div>
        ),
        key: 'Bilder'
      },
      {
        heading: 'Beskrivelse',
        selectedValue: observationForm.data.description ? ` : ${observationForm.data.description}` : '',
        body: (
          <div>
            <TextField
              field="description"
              updateDataField={formActions.updateDataField}
              value={observationForm.data.description}
              rows={5}
            />
          </div>
        ),
        key: 'Beskrivelse'
      }
    ];
  }

  getFormElement() {
    const { observationForm } = this.props;
    const standardFormElements = this.getStandardFormElements();
    let elements;

    switch (observationForm.category) {
      case observationCategory.DOG: {
        this.formStructure = dogFormStructure;
        elements = this.getFormElements(dogFormStructure);
        break;
      }
      case observationCategory.OTHER:
        this.formStructure = standardFormElements;
        return standardFormElements;
      case observationCategory.INJUERED_SHEEP: {
        this.formStructure = injuredSheepFormStructure;
        elements = this.getFormElements(injuredSheepFormStructure);
        break;
      }
      case observationCategory.REINDEER: {
        this.formStructure = reindeerFormStructure;
        elements = this.getFormElements(reindeerFormStructure);
        break;
      }
      case observationCategory.PREDATOR: {
        this.formStructure = predatorFormStructure;
        elements = this.getFormElements(predatorFormStructure);
        break;
      }
      case observationCategory.SHEEP: {
        this.formStructure = sheepFormStructure;
        elements = this.getFormElements(sheepFormStructure);
        break;
      }
      case observationCategory.DEAD_SHEEP:
        this.formStructure = deadSheepFormStructure;
        elements = this.getFormElements(deadSheepFormStructure);
        break;
      case observationCategory.WOOL:
        this.formStructure = woolFormStructure;
        elements = this.getFormElements(woolFormStructure);
        break;
      default:
        elements = [];
        break;
    }
    // Add Image and Description
    return elements.concat(standardFormElements);
  }

  getFlatFormElementStructure(formDataStructure1) {
    const { elements } = formDataStructure1;
    const flat = this.getFlatFormElementList(elements);

    const isGroup = this.props.observationForm.data.category === SHEEP_GROUP;
    const hasInjury = this.props.observationForm.data.hasInjury === 'Ja';

    if (isGroup && hasInjury) {
      const numberOfInjuredSheep = this.props.observationForm.data.count;
      let temp = [];

      for (let i = 0; i < numberOfInjuredSheep; i++) {
        const t = this.getFlatFormElementList(getInjuryStructure(i));
        temp = temp.concat(t);
      }

      flat.splice((flat.length - 1), 0, ...temp);   // Add to lengt-1 because we want to show contact after injury questions
    }

    return {
      ...formDataStructure1,
      elements: flat
    };
  }

  /**
   * Returns a flat form structure from the form structure hierarchy.
   * @param structure
   * @returns {*}
   */
  getFlatFormElementList(elements) {
    let isFlat = true;
    for (let i = 0; i < elements.length; i++) {
      if (elements[i].subcategories !== undefined) {
        isFlat = false;
      }
    }

    if (isFlat) {
      return elements;
    }

    let flat = [];
    for (let i = 0; i < elements.length; i++) {
      if (elements[i].subcategories === undefined) {
        flat.push(elements[i]);
      } else {
        flat.push(elements[i]);
        const field = elements[i].formInfo.field;
        const category = this.props.observationForm.data[field];
        const categoryElements = elements[i].subcategories[category];

        if (categoryElements !== undefined) {
          const l = categoryElements instanceof Array ? categoryElements : [categoryElements];
          const res = this.getFlatFormElementList(l);
          flat = flat.concat(res);
        }
      }
    }
    return flat;
  }

  getFormElements(formDataStructure1) {
    let formDataStructure = this.getFlatFormElementStructure(formDataStructure1);


    const { data } = this.props.observationForm;
    const { category } = this.props.observationForm.data;
    const { categoryElement } = formDataStructure;

    if (formDataStructure.hasCategories) {
      const elements = [{
        heading: categoryElement.heading,
        selectedValue: getHeading(categoryElement, data),
        body: this.getBody(categoryElement.formInfo, true),
        key: categoryElement.formInfo.key
      }];
      if (category && category.length > 0) {
        const associatedElements = formDataStructure[category].map(element => {
          return {
            heading: element.heading,
            selectedValue: getHeading(element, data),
            body: this.getBody(element.formInfo),
            key: element.formInfo.key ? element.formInfo.key : element.formInfo.field
          };
        });
        return elements.concat(associatedElements);
      }
      return elements;
    } else {
      return formDataStructure.elements.map(element => {
        return {
          heading: element.heading,
          selectedValue: getHeading(element, data),
          body: this.getBody(element.formInfo),
          key: element.formInfo.key ? element.formInfo.key : element.formInfo.field
        };
      });
    }
  }

  getPanelDataList() {
    const { observationForm } = this.props;
    const list = [
      {
        heading: 'Kategori',
        selectedValue: (observationForm.category.length > 0 ? ': ' + observationForm.category : null),
        body: (
          <div>
            <CategorySelection
              observationFormActions={this.props.observationFormActions}
              observationForm={this.props.observationForm}
            />
          </div>
        ),
        key: 'kategori'
      }
    ];
    return list.concat(this.getFormElement());
  }

  getCollapsiblePanelElement(element, index) {
    const selectedValue = element.selectedValue !== null ? element.selectedValue : '';

    const expended = () => {
      if (this.isEdit() && this.firstTimeOpeningCategoryPanel && this.isSelectedPanel(index)) {
        this.firstTimeOpeningCategoryPanel = false;
        return false;
      } else {
        return this.isSelectedPanel(index);
      }
    };

    return (
      <CollapsiblePanel
        heading={element.heading}
        selectedValue={selectedValue}
        body={element.body}
        expanded={expended()}
        onClick={() => {
          this.firstTimeOpeningCategoryPanel = false;
          this.onPanelClick(index);
        }}
        key={element.key}
      />
    );
  }

  getSaveButton() {
    return (
      <button
        onClick={this.saveObservation}
        className="save-button"
      >
        <div className="text">
          <i className="fa fa-check-circle-o" /> Lagre
        </div>
      </button>
    );
  }

  saveObservation() {
    const { updateObservationData } = this.props.observationRoundActions;
    const registrationPointId = this.props.registrationPointId;
    const observationId = this.props.observationId;
    const data = { ...this.props.observationForm.data };

    const type = data.type.length > 0 ? data.type : data.category;
    data.type = getCategory(type);

    const formattedData = getFormattedObservationData(data);

    updateObservationData(registrationPointId, observationId, formattedData);

    this.clearData();
    this.props.onSubmit(observationId, formattedData);

    transition.animateToLeftPosition('new-observation', '100vw', () => {
      this.props.observationFormActions.resetForm();
      this.props.routerActions.toggleNewObservationView();
    });
  }

  handleChangeCategory(field, data) {
    const formActions = this.props.observationFormActions;
    if (this.props.observationForm.index === 1) {
      formActions.updateDataField(field, data);
      formActions.setIndex(2);
      formActions.setDisplayIndex(2);
    }
  }

  goToNextPanel(field = '') {
    const fieldsHasBeenCleared = this.clearDataElements(getKeysToDelete(this.formStructure, field));
    const formActions = this.props.observationFormActions;

    if (!this.props.edit && this.isNewestElement()) {
      formActions.incrementDisplayIndex();
    } else if (fieldsHasBeenCleared) {
      formActions.setDisplayIndex(this.props.observationForm.index + 1);
    }
    formActions.incrementIndex();
  }

  clearDataElements(elementsToDelete) {
    const { data } = this.props.observationForm;
    const keysToClear = Object.keys(data).filter(element => {
      return arrayContains(elementsToDelete, element);
    });

    let elementsHasBeenCleared = false;
    keysToClear.forEach(key => {
      if (typeof data[key] === 'string') {
        if (data[key] !== '') {
          this.props.observationFormActions.updateDataField(key, '');
          elementsHasBeenCleared = true;
        }
      }
    });

    return elementsHasBeenCleared;
  }

  clearData() {
    const { data } = this.props.observationForm;
    const keysToClear = Object.keys(data).filter(element => {
      return element !== 'category';
    });
    return keysToClear.forEach(key => {
      if (typeof data[key] === 'string') {
        this.props.observationFormActions.updateDataField(key, '');
      } else if (data[key] instanceof Array) {
        // this.props.observationFormActions.updateDataField(key, []);
      }
    });
  }

  isSelectedPanel(index) {
    return this.props.observationForm.index === index;
  }

  navigateBack() {
    this.props.observationFormActions.resetForm();
    transition.animateToLeftPosition('new-observation', '100vw', () => {
      this.props.routerActions.toggleNewObservationView();
    });
  }

  isNewestElement() {
    const { observationForm } = this.props;
    return observationForm.displayIndex === observationForm.index;
  }

  showNextButton() {
    return !this.props.edit && !this.isLastElement() && this.isNewestElement();
  }

  showSaveButton() {
    const { observationForm } = this.props;
    const { displayIndex } = observationForm;
    const indexHiddenSaveButton = ('category' in observationForm.data) ? 1 : 0;
    return displayIndex > indexHiddenSaveButton;
  }


  isLastElement() {
    const { observationForm } = this.props;
    return observationForm.index === getNumberOfPanels(observationForm);
  }

  changeSubCategory() {
    const formActions = this.props.observationFormActions;
    formActions.setDisplayIndex(2);
    formActions.setIndex(2);
    this.clearData();
  }

  willDisplayPanel(element, index) {
    return index <= this.props.observationForm.displayIndex;
  }

  render() {
    const displayElements = this.getPanelDataList().filter(this.willDisplayPanel);
    const panels = displayElements.map(this.getCollapsiblePanelElement);

    return (
      <div>
        <div id="new-observation">
          <NavigationBar
            onClick={this.navigateBack}
            save={this.saveObservation}
          />
          <div className="new-observation-container">
            {panels}
            <div className="save-button-container">
              {this.showSaveButton() ? this.getSaveButton() : null}
            </div>
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    map: state.map,
    observationForm: state.observationForm,
    observationRound: state.observationRound,
    appRouter: state.appRouter
  };
}

function mapDispatchToProps(dispatch) {
  return {
    mapActions: bindActionCreators(mapActions, dispatch),
    observationFormActions: bindActionCreators(observationFormActions, dispatch),
    observationRoundActions: bindActionCreators(ObservationRoundActions, dispatch),
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(NewObservation);
