import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import shortid from 'shortid';
import * as mapActions from 'src/actions/mapActions';
import municipalities from '../utilities/municipalities.js';
import * as ObservationRoundActions from '../actions/observationRoundActions.js';
import * as routerActions from '../actions/routerActions';
import * as transition from '../utilities/transitions.js';
import NumberPicker from '../common/NumberPicker.jsx';
import weather from '../constants/weather.js';
import NavigationBar from '../common/NavigationBar.jsx';


class NewObservationRound extends Component {
  constructor(props) {
    super(props);
    this.state = ({
      farmNumber: '',
      buildingNumber: '',
      municipality: '',
      name: '',
      description: ''
    });
    this.updateNumberOfParticipants = this.updateNumberOfParticipants.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onWeatherChange = this.onWeatherChange.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.onEdit = this.onEdit.bind(this);
    this.navigateBack = this.navigateBack.bind(this);
  }

  componentWillMount() {
    this.setState(this.props.observationRound);
    document.addEventListener('backbutton', this.navigateBack, false);
  }

  componentDidMount() {
    transition.animateRightToLeft('new-observation-round');
  }

  componentWillUnmount() {
    document.removeEventListener('backbutton', this.navigateBack, false);
  }

  onSubmit(e) {
    e.preventDefault();
    const data = this.state;
    data.lastSavedTimestamp = Date.now();
    if (!this.props.roundIsActive) {
      data.createdTimestamp = Date.now();
      data.id = shortid.generate();
    }
    this.props.observationRoundActions.saveObservationRoundMetadata(data);
    transition.animateToLeftPosition('new-observation-round', '100vw', () => {
      this.props.routerActions.toggleNewObservationRoundView();
      if (!this.props.roundIsActive) {
        this.props.onSubmit();
      }
    });
  }

  onEdit(e) {
    e.preventDefault();
    const data = {
      ...this.state,
      lastSavedTimestamp: Date.now()
    };
    this.props.observationRoundActions.saveObservationRoundMetadata(data);
    transition.animateToLeftPosition('new-observation-round', '100vw', () => {
      this.props.routerActions.toggleNewObservationRoundView();
    });
  }

  onChange(event) {
    this.setState({
      [event.target.name]: event.target.value
    });
  }

  onWeatherChange(event) {
    this.setState({
      ...this.state,
      weather: event.target.value
    });
  }

// eslint-disable-next-line class-methods-use-this
  getTime() {
    const date = new Date();
    return date.getTime();
  }

  // eslint-disable-next-line class-methods-use-this
  getWeatherOptions() {
    return weather.map(value => {
      return (
        <option
          value={value}
          key={value}
        >
          {value}
        </option>
      );
    });
  }

  updateNumberOfParticipants(numberOfParticipants) {
    this.setState({
      numberOfParticipants
    });
  }

  navigateBack() {
    transition.animateToLeftPosition('new-observation-round', '100vw', () => {
      this.props.routerActions.toggleNewObservationRoundView();
    });
  }

  render() {
    const roundIsActive = this.props.roundIsActive;
    const submitText = roundIsActive ? 'Lagre' : 'Start tilsynstur';
    const heading = roundIsActive ? 'Turinfo' : 'Ny tilsynstur';
    return (
      <div id="new-observation-round">
        <NavigationBar
          label={heading}
          onClick={this.navigateBack}
        />
        <form className="new-observation-round-form" onSubmit={this.state.roundIsActive ? this.onEdit : this.onSubmit}>
          <div className="form-group">
            <label className={`label${this.state.name !== '' ? ' reposition' : ''}`}>Ditt navn:</label>
            <input
              type="text"
              name="name"
              value={this.state.name}
              onChange={this.onChange}
              className={`form-control${this.state.name !== '' ? ' reposition' : ''}`}
              required
            />
          </div>

          <div className="form-group">
            <label className={`label${this.state.farmNumber !== '' ? ' reposition' : ''}`}>Gårdsnummer:</label>
            <input
              name="farmNumber"
              type="number"
              value={this.state.farmNumber}
              onChange={this.onChange}
              className={`form-control${this.state.farmNumber !== '' ? ' reposition' : ''}`}
              required
            />
          </div>
          <div className="form-group">
            <label className={`label${this.state.buildingNumber !== '' ? ' reposition' : ''}`}>Bruksnummer:</label>
            <input
              type="number"
              name="buildingNumber"
              value={this.state.buildingNumber}
              onChange={this.onChange}
              className={`form-control${this.state.buildingNumber !== '' ? ' reposition' : ''}`}
            />
          </div>
          <div className="form-group">
            <label className={`label${this.state.municipality !== '' ? ' reposition' : ''}`}>Kommune:</label>
            <input
              type="text"
              name="municipality"
              value={this.state.municipality}
              list="muncipalities"
              onChange={this.onChange}
              className={`form-control${this.state.municipality !== '' ? ' reposition' : ''}`}
              required
            />
            <i className="fa fa-search" />
            <datalist id="muncipalities">
              {municipalities.map(municipality => {
                return <option key={municipality.id} value={municipality.name} />;
              })}
            </datalist>
          </div>
          <div className="form-group">
            <label>Antall deltagere:</label>
            <NumberPicker
              value={this.state.numberOfParticipants}
              onChange={this.updateNumberOfParticipants}
              bounds={[1, 100]}
            />
          </div>
          <div className="form-group">
            <label>Vær:</label>
            <select
              className="form-control"
              onChange={this.onWeatherChange}
              value={this.state.weather}
            >
              <option value="" defaultValue disabled >Velg</option>
              {this.getWeatherOptions()}
            </select>
          </div>
          <div className="form-group">
            <label className={`description-label${this.state.description !== '' ? ' reposition' : ''}`}>Beskrivelse:</label>
            <textarea
              value={this.state.description}
              onChange={this.onChange}
              className={`form-control${this.state.description !== '' ? ' reposition' : ''}`}
              rows="4"
              name="description"
            />
          </div>
          <div className="form-group submit-container">
            <button
              type="submit"
              className="btn btn-default form-control submit-button"
            >
              <i className="width-helper" />
              <div className="text">{submitText}</div>
              <i className="fa fa-angle-right button-icon" />
            </button>
          </div>
        </form>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    observationRound: state.observationRound,
    appRouter: state.router,
    map: state.map
  };
}

function mapDispatchToProps(dispatch) {
  return {
    observationRoundActions: bindActionCreators(ObservationRoundActions, dispatch),
    routerActions: bindActionCreators(routerActions, dispatch),
    mapActions: bindActionCreators(mapActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(NewObservationRound);
