/* eslint-disable react/prop-types */
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as transition from '../utilities/transitions.js';
import * as mapActions from '../actions/mapActions';
import * as routerActions from '../actions/routerActions';
import * as observationFormActions from '../actions/observationFormActions';
import * as ObservationRoundActions from '../actions/observationRoundActions.js';
import CollapsiblePanel from './newObservation/CollapsiblePanel.jsx';
import Alternatives from '../containers/newObservation/panelBodyElement/Alternatives.jsx';
import Counter from './newObservation/panelBodyElement/Counter.jsx';
import Images from '../containers/newObservation/panelBodyElement/Images.jsx';
import TextField from '../containers/newObservation/panelBodyElement/TextField.jsx';
import CategorySelection from './newObservation/CategorySelection.jsx';
import * as observationCategory from '../constants/observation/NewObservationCategories';
import SheepColor from './newObservation/panelBodyElement/SheepColor.jsx';
import { getUpdateDataFieldFunction, getHeading, getNumberOfPanels, getCategory } from '../utilities/newObservationHelper';
import NavigationBar from './newObservation/NavigationBar.jsx';
import { deadSheepFormStructure } from '../constants/observation/deadSheep';
import { dogFormStructure } from '../constants/observation/dog';
import { injuredSheepFormStructure } from '../constants/observation/injuredSheep';
import { predatorFormStructure } from '../constants/observation/predator';
import { reindeerFormStructure } from '../constants/observation/reindeer';
import { MULTIPLE_GROUPS, sheepFormStructure, getSheepGroup, categories } from '../constants/observation/sheep';
import { woolFormStructure } from '../constants/observation/wool';
import { ALTERNATIVES, COUNTER, SHEEP_COLOR, TEXT_FIELD } from '../constants/observation/formElementtTypes';
import { MAIN_MAP } from '../constants/paths';

class NewObservation extends Component {
  constructor(props) {
    super(props);

    this.isSelectedPanel = this.isSelectedPanel.bind(this);
    this.onPanelClick = this.onPanelClick.bind(this);
    this.changeSubCategory = this.changeSubCategory.bind(this);
    this.goToNextPanel = this.goToNextPanel.bind(this);
    this.handleChangeCategory = this.handleChangeCategory.bind(this);
    this.willDisplayPanel = this.willDisplayPanel.bind(this);
    this.getCollapsiblePanelElement = this.getCollapsiblePanelElement.bind(this);
    this.saveObservation = this.saveObservation.bind(this);
    this.navigateBack = this.navigateBack.bind(this);
    this.saveObservation = this.saveObservation.bind(this);
  }

  componentWillMount() {
    const { observationId, registrationPointId } = this.props.map.openObservationIds;
    const { registrationPoints } = this.props.observationRound;
    let observationData = { type: '' };
    try {
      observationData = registrationPoints[registrationPointId].observations[observationId].data;
    } catch (e) {
      console.log(e);
    }

    const isEdit = Object.keys(observationData).length > 3 || observationData.type.length > 0;

    if (isEdit) {
      const state = { ...this.props.observationForm };
      state.data = observationData;
      const type = observationData.type.length > 0 ? observationData.type : observationData.category;
      state.category = getCategory(type);
      state.index = -1;
      state.displayIndex = getNumberOfPanels(state) + 1;
      state.edit = true;

      this.props.observationFormActions.setFormState(state);
    } else {
      this.props.observationFormActions.setIndex(0);
    }
  }

  componentDidMount() {
    transition.animateRightToLeft('new-observation');
  }

  onPanelClick(index) {
    let i = index;
    if (this.isSelectedPanel(i)) {
      i = -1;
    }
    this.props.observationFormActions.setCurrentPanel(i);
  }

  getSheepAlternativeValue(formInfo, formData) {
    if (formInfo.field === 'color' && (formData.category === categories[0] || formData.category === categories[1])) {
      let colorObject = {};
      if (formData.category === categories[0]) {
        colorObject = formData.groups[0].ewes;
      } else if (formData.category === categories[1]) {
        colorObject = formData.groups[0].lamb;
      }

      if (colorObject.black > 0) {
        return 'Svart';
      } else if (colorObject.white > 0) {
        return 'Hvit';
      } else if (colorObject.blackAndWhite > 0) {
        return 'Svart og hvit';
      }
    }
    return null;
  }

  getBody(formInfo, isCategory = false) {
    const formActions = this.props.observationFormActions;
    const formData = this.props.observationForm.data;
    const updateDataField = getUpdateDataFieldFunction(formData.category, formInfo, formActions, this.handleChangeCategory);

    const sheepValue = this.getSheepAlternativeValue(formInfo, formData);
    switch (formInfo.type) {
      case ALTERNATIVES:
        console.log('_________________________formInfo')
        console.log(formInfo)
        if (formInfo.showNextCount) {
          console.log('num of extra fields if yes', formInfo.showNextCount);
        }
        return (
          <div>
            <Alternatives
              alternatives={formInfo.alternatives}
              field={formInfo.field}
              updateDataField={updateDataField}
              onSubmit={isCategory ? this.changeSubCategory : this.goToNextPanel}
              value={sheepValue !== null ? sheepValue : formData[formInfo.field]}
              icons={formInfo.icons !== undefined ? formInfo.icons : {}}
            />
          </div>
        );
      case TEXT_FIELD:
        return (
          <div>
            <TextField
              field={formInfo.field}
              updateDataField={formActions.updateDataField}
              value={formData[formInfo.field]}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      case SHEEP_COLOR: {
        return (
          <div>
            <SheepColor
              field={formInfo.field}
              groupIndex={formInfo.groupIndex}
              updateDataField={updateDataField}
              value={formData.groups[formInfo.groupIndex][formInfo.field]}
              key={formInfo.groupIndex}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      }
      case COUNTER: {
        return (
          <div>
            <Counter
              field={formInfo.field}
              updateDataField={formActions.updateDataField}
              formActions={formActions}
              value={formData[formInfo.field]}
            />
            {this.showNextButton() ? this.getNextButton() : null}
          </div>
        );
      }
      default:
        return null;
    }
  }

  getNextButton() {
    return (
      <button
        onClick={this.goToNextPanel}
        className="next-button"
      >
        <div className="text">
          Neste
        </div>
      </button>
    );
  }

  getStandardFormElements() {
    const formActions = this.props.observationFormActions;
    const { observationForm } = this.props;
    const nextButton = this.getNextButton();

    return [
      {
        heading: 'Bilder',
        body: (
          <div>
            <Images observationFormActions={this.props.observationFormActions} observationForm={this.props.observationForm} />
            {this.showNextButton() ? nextButton : null}
          </div>
        ),
        key: 'Bilder'
      },
      {
        heading: 'Beskrivelse',
        selectedValue: observationForm.data.description ? ` : ${observationForm.data.description}` : '',
        body: (
          <div>
            <TextField
              field="description"
              updateDataField={formActions.updateDataField}
              value={observationForm.data.description}
              rows={5}
            />
            {this.showNextButton() ? nextButton : null}
          </div>
        ),
        key: 'Beskrivelse'
      }
    ];
  }

  getFormElement() {
    const { observationForm } = this.props;
    const standardFormElements = this.getStandardFormElements();
    let elements;

    switch (observationForm.category) {
      case observationCategory.DOG: {
        elements = this.getFormElements(dogFormStructure);
        break;
      }
      case observationCategory.OTHER:
        return standardFormElements;
      case observationCategory.INJUERED_SHEEP: {
        elements = this.getFormElements(injuredSheepFormStructure);
        break;
      }
      case observationCategory.REINDEER: {
        elements = this.getFormElements(reindeerFormStructure);

        break;
      }
      case observationCategory.PREDATOR: {
        elements = this.getFormElements(predatorFormStructure);
        break;
      }
      case observationCategory.SHEEP: {
        const formStructure = { ...sheepFormStructure };
        const multipleGroups = formStructure[MULTIPLE_GROUPS].concat(getSheepGroup(0));
        formStructure[MULTIPLE_GROUPS] = multipleGroups;
        elements = this.getFormElements(formStructure);
        break;
      }
      case observationCategory.DEAD_SHEEP:
        elements = this.getFormElements(deadSheepFormStructure);
        break;
      case observationCategory.WOOL:
        elements = this.getFormElements(woolFormStructure);
        break;
      default:
        elements = [];
        break;
    }
    // Add Image and Description
    return elements.concat(standardFormElements);
  }

  getFormElements(formDataStructure) {
    const { data } = this.props.observationForm;
    const { category } = this.props.observationForm.data;
    const { categoryElement } = formDataStructure;

    if (category === MULTIPLE_GROUPS) {
      const formStructure = { ...sheepFormStructure };
      const multipleGroups = formStructure[MULTIPLE_GROUPS].concat(getSheepGroup(0, this.props.observationForm.data));
      formStructure[MULTIPLE_GROUPS] = multipleGroups;
      formDataStructure = formStructure;
    }


    if (formDataStructure.hasCategories) {
      const elements = [{
        heading: categoryElement.heading,
        selectedValue: getHeading(categoryElement, data),
        body: this.getBody(categoryElement.formInfo, true),
        key: categoryElement.formInfo.key
      }];
      if (category && category.length > 0) {
        const associatedElements = formDataStructure[category].map(element => {
          return {
            heading: element.heading,
            selectedValue: getHeading(element, data),
            body: this.getBody(element.formInfo),
            key: element.formInfo.key ? element.formInfo.key : element.formInfo.field
          };
        });
        return elements.concat(associatedElements);
      }
      return elements;
    } else {
      return formDataStructure.elements.map(element => {
        return {
          heading: element.heading,
          selectedValue: getHeading(element, data),
          body: this.getBody(element.formInfo),
          key: element.formInfo.key ? element.formInfo.key : element.formInfo.field
        };
      });
    }
  }

  getPanelDataList() {
    const { observationForm } = this.props;
    const list = [
      {
        heading: 'Kategori',
        selectedValue: (observationForm.category.length > 0 ? ': ' + observationForm.category : null),
        body: (
          <div>
            <CategorySelection
              observationFormActions={this.props.observationFormActions}
              observationForm={this.props.observationForm}
            />
          </div>
        ),
        key: 'kategori'
      }
    ];
    return list.concat(this.getFormElement());
  }

  getCollapsiblePanelElement(element, index) {
    console.log(element);
    return (
      <CollapsiblePanel
        heading={element.heading}
        selectedValue={element.selectedValue ? element.selectedValue : null}
        body={element.body}
        expanded={this.isSelectedPanel(index)}
        onClick={() => {
          this.onPanelClick(index);
        }}
        key={element.key}
      />
    );
  }

  getSaveButton() {
    return (
      <button
        onClick={this.saveObservation}
        className="save-button"
      >
        <div className="text">
          <i className="fa fa-check-circle-o" /> Lagre
        </div>
      </button>
    );
  }

  saveObservation() {
    const { updateObservationData } = this.props.observationRoundActions;
    const { observationId, registrationPointId } = this.props.map.openObservationIds;
    const data = { ...this.props.observationForm.data };
    const type = data.type.length > 0 ? data.type : data.category;
    data.type = getCategory(type);

    updateObservationData(registrationPointId, observationId, data);

    transition.animateToLeftPosition('new-observation', '100vw', () => {
      this.props.router.push(MAIN_MAP);
      this.props.observationFormActions.resetForm();
    });
  }

  handleChangeCategory(field, data) {
    const formActions = this.props.observationFormActions;
    if (this.props.observationForm.index === 1) {
      formActions.updateDataField(field, data);
      formActions.setIndex(2);
      formActions.setDisplayIndex(2);
    }
  }

  goToNextPanel() {
    if (!this.props.edit && this.isNewestElement()) {
      const formActions = this.props.observationFormActions;
      formActions.incrementDisplayIndex();
      formActions.incrementIndex();
    }
  }

  clearData() {
    const { data } = this.props.observationForm;
    const keysToClear = Object.keys(data).filter(element => {
      return element !== 'category';
    });
    return keysToClear.forEach(key => {
      if (typeof data[key] === 'string') {
        this.props.observationFormActions.updateDataField(key, '');
      } else if (data[key] instanceof Array) {
        // this.props.observationFormActions.updateDataField(key, []);
      }
    });
  }

  isSelectedPanel(index) {
    return this.props.observationForm.index === index;
  }

  navigateBack() {
    this.props.observationFormActions.resetForm();
    transition.animateToLeftPosition('new-observation', '100vw', () => {
      this.props.routerActions.toggleNewObservationView()
    });
  }

  isNewestElement() {
    const { observationForm } = this.props;
    return observationForm.displayIndex === observationForm.index;
  }

  showNextButton() {
    return !this.props.edit && !this.isLastElement() && this.isNewestElement();
  }

  showSaveButton() {
    const { observationForm } = this.props;
    const { displayIndex } = observationForm;
    const indexHiddenSaveButton = ('category' in observationForm.data) ? 1 : 0;
    return displayIndex > indexHiddenSaveButton;
  }


  isLastElement() {
    const { observationForm } = this.props;
    return observationForm.index === getNumberOfPanels(observationForm);
  }

  changeSubCategory() {
    const formActions = this.props.observationFormActions;
    formActions.setDisplayIndex(2);
    formActions.setIndex(2);
    this.clearData();
  }

  willDisplayPanel(element, index) {
    return index <= this.props.observationForm.displayIndex;
  }

  render() {
    const displayElements = this.getPanelDataList().filter(this.willDisplayPanel);
    const panels = displayElements.map(this.getCollapsiblePanelElement);

    return (
      <div>
        <div id="new-observation">
          <NavigationBar
            onClick={this.navigateBack}
            save={this.saveObservation}
          />
          <div className="new-observation-container">
            {panels}
            <div className="save-button-container">
              {this.showSaveButton() ? this.getSaveButton() : null}
            </div>
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    map: state.map,
    observationForm: state.observationForm,
    observationRound: state.observationRound,
    appRouter: state.appRouter
  };
}

function mapDispatchToProps(dispatch) {
  return {
    mapActions: bindActionCreators(mapActions, dispatch),
    observationFormActions: bindActionCreators(observationFormActions, dispatch),
    observationRoundActions: bindActionCreators(ObservationRoundActions, dispatch),
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(NewObservation);
