import React, { Component } from 'react';
import Switch from 'react-switch';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as routerActions from '../actions/routerActions';
import * as transition from '../utilities/transitions.js';
import NavigationBar from '../common/NavigationBar.jsx';
import { saveSettings, getSettings } from '../utilities/save';


class SettingsView extends Component {
  constructor(props) {
    super(props);

    const { userTrackingIsOn, automaticUploadIsEnabled } = getSettings();

    this.state = {
      userTrackingIsOn,
      automaticUploadIsEnabled
    };
    this.navigateBack = this.navigateBack.bind(this);
    this.onUserTrackingToggle = this.onUserTrackingToggle.bind(this);
    this.onAutomaticUploadToggle = this.onAutomaticUploadToggle.bind(this);
  }

  componentWillMount() {
    document.addEventListener('backbutton', this.navigateBack, false);
  }

  componentDidMount() {
    transition.animateRightToLeft('settings-view');
  }

  componentWillUnmount() {
    document.removeEventListener('backbutton', this.navigateBack, false);
  }

  onUserTrackingToggle(value) {
    const settings = {
      ...this.state,
      userTrackingIsOn: value
    };
    this.props.onUserTrackingToggle(value);
    saveSettings(settings);
    this.setState(settings);
  }

  onAutomaticUploadToggle(value) {
    console.log(value);
    const settings = {
      ...this.state,
      automaticUploadIsEnabled: value
    };
    this.props.onAutomaticUploadToggle(value);
    saveSettings(settings);
    this.setState(settings);
  }

  navigateBack() {
    transition.animateToLeftPosition('settings-view', '100vw', () => {
      this.props.routerActions.toggleSettingsView();
    });
  }

  render() {
    return (
      <div id="settings-view">
        <NavigationBar
          label="Innstillinger"
          onClick={this.navigateBack}
        />
        <div className="settings-body">
          <div className="option">
            <label >
              Automatisk opplasting av data
            </label>
            <Switch
              checked={this.state.automaticUploadIsEnabled}
              className="react-switch"
              onChange={this.onAutomaticUploadToggle}
              id="normal-switch"
            />
          </div>
          <div className="option">
            <label >
              Vis gålinje
            </label>
            <Switch
              checked={this.state.userTrackingIsOn}
              className="react-switch"
              onChange={this.onUserTrackingToggle}
              id="normal-switch"
            />
          </div>
          <div className="option">
            <label>
              {`Lagrede kartdata: ${this.props.totalStorage} filer`}
            </label>
            <button
              className={`delete-button${this.props.totalStorage === 0 ? ' hide' : ''}`}
              onClick={() => this.props.onDeleteTiles()}
            >
              Slett
            </button>
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    appRouter: state.router
  };
}

function mapDispatchToProps(dispatch) {
  return {
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(SettingsView);
