/* eslint-disable react/require-default-props,react/forbid-prop-types,jsx-a11y/no-static-element-interactions */
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import * as routerActions from 'src/actions/routerActions';
import * as transition from '../../utilities/transitions';
import * as swipeable from '../../utilities/swipeableMenu.js';


class NavigationMenu extends Component {

  constructor(props) {
    super(props);
    this.state = {
      show: false
    };
    this.onNewObservationRound = this.onNewObservationRound.bind(this);
    this.closeNavigationMenu = this.closeNavigationMenu.bind(this);
    this.onSavedRounds = this.onSavedRounds.bind(this);
    this.onSettingsClick = this.onSettingsClick.bind(this);
    this.onFinishRoundClick = this.onFinishRoundClick.bind(this);
  }

  componentDidMount() {
    swipeable.init('navigation-menu', this.closeNavigationMenu);
    transition.animateToLeftPosition('navigation-menu', 0);
    this.setState({
      show: true
    });
    document.addEventListener('backbutton', this.closeNavigationMenu, false);
  }

  componentWillUnmount() {
    swipeable.removeEventListeners(this.closeNavigationMenu);
    document.removeEventListener('backbutton', this.closeNavigationMenu, false);
  }

  onNewObservationRound() {
    this.props.routerActions.toggleNewObservationRoundView();
    this.closeNavigationMenu();
  }

  onSettingsClick() {
    this.props.routerActions.toggleSettingsView();
    this.closeNavigationMenu();
  }

  onFinishRoundClick() {
    this.props.routerActions.toggleFinishRoundView();
    this.closeNavigationMenu();
  }

  onSavedRounds() {
    this.props.routerActions.toggleSavedRoundsView();
    transition.animateToLeftPosition('navigation-menu', '-100vw', () => {
      this.props.routerActions.toggleNavigationMenuView();
    });
  }

  closeNavigationMenu(method = () => {}) {
    this.setState({
      show: false
    });
    transition.animateToLeftPosition('navigation-menu', '-100vw', () => {
      this.props.routerActions.toggleNavigationMenuView();
      method();
    });
  }

  render() {
    const observationRoundText = this.props.roundIsActive ? 'Rediger turinfo' : 'Ny tilsynstur';
    return (
      <div>
        <div className={`shadow-overlay${this.state.show ? ' show' : ''}`} onClick={this.closeNavigationMenu} />
        <div id="navigation-menu">
          <div className="navigation-menu">
            <div className="banner">
              <div className="banner-text">
                Lambo
              </div>
            </div>
            <div className="navigation-options">
              <div onClick={this.onFinishRoundClick} className={`navigation-option${this.props.roundIsActive ? '' : ' hide'}`}>
                <div className="option-icon-container">
                  <i className="option-icon fa fa-check-circle" />
                </div>
                <div className="option-text">
                  Fullfør tilsynstur
                </div>
                <i className="right-icon fa fa-angle-right" />
              </div>
              <div onClick={this.onNewObservationRound} className="navigation-option">
                <div className="option-icon-container">
                  <img className="option-icon new-round" src="img/vectors/NewObservationForm/ewe_black.svg" />
                </div>
                <div className="option-text">
                  {observationRoundText}
                </div>
                <i className="right-icon fa fa-angle-right" />
              </div>
              <div className="separator" />
              <div className="navigation-option" onClick={this.onSavedRounds}>
                <div className="option-icon-container">
                  <i className="option-icon fa fa-save" />
                </div>
                <div className="option-text">
                  Lagrede tilsynsturer
                </div>
                <i className="right-icon fa fa-angle-right" />
              </div>
              <div className="navigation-option" onClick={() => this.closeNavigationMenu(this.props.onDownloadMapClick)}>
                <div className="option-icon-container">
                  <i className="option-icon fa fa-map" />
                </div>
                <div className="option-text">
                  Kartdata
                </div>
                <i className="right-icon fa fa-angle-right" />
              </div>
              <div className="separator" />
              <div className="navigation-option" onClick={this.onSettingsClick}>
                <div className="option-icon-container">
                  <i className="option-icon fa fa-cog" />
                </div>
                <div className="option-text">
                  Innstillinger
                </div>
                <i className="right-icon fa fa-angle-right" />
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    appRouter: state.router
  };
}

function mapDispatchToProps(dispatch) {
  return {
    routerActions: bindActionCreators(routerActions, dispatch)
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(NavigationMenu);
