/* eslint-disable jsx-a11y/no-static-element-interactions,react/sort-comp */
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import shortid from 'shortid';
import SmoothCollapse from 'react-smooth-collapse';

class CollapsiblePanel extends Component {

  scrollToView(id) {
    document.getElementById(id).scrollIntoView();
  }

  render() {
    const { heading, selectedValue, body } = this.props;
    const expanded = this.props.expanded;
    const id = shortid.generate();
    return (
      <div className="collapsible-panel" id={id}>
        <div className={`panel-header${expanded ? ' expanded' : ''}`} onClick={this.props.onClick}>
          <div className="panel-header-name">{heading}</div>
          <div className="panel-header-value">{selectedValue}</div>
          <i
            className={'fa ' + (expanded ? 'fa-angle-down' : 'fa-angle-right')}
          />
        </div>
        <SmoothCollapse expanded={expanded} onChangeEnd={() => { this.scrollToView(id); }}>
          <div className="panel-body">
            {body}
          </div>
        </SmoothCollapse>
      </div>
    );
  }
}

CollapsiblePanel.defaultProps = {
  onClick: () => { console.error('No onClick prop provided!'); },
  expanded: false
};

if (__DEV__) {
  CollapsiblePanel.propTypes = {
    heading: PropTypes.string.isRequired,
    body: PropTypes.element.isRequired,
    onClick: PropTypes.func,
    expanded: PropTypes.bool
  };
}

export default CollapsiblePanel;
