/* eslint-disable react/forbid-prop-types */
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import Button from '../../../common/Button.jsx';
import { LAZY_TOUCH_DELAY } from '../../../constants/lazyTouch';

class Alternatives extends Component {
  constructor(props) {
    super(props);
    this.state = {
      selected: -1,
      isClickable: false
    };
    this.onSelect = this.onSelect.bind(this);
  }

  componentDidMount() {
    const enableButton = () => {
      this.setState({
        isClickable: true
      });
    };
    setTimeout(enableButton, LAZY_TOUCH_DELAY);
  }

  onSelect(alternative) {
    if (this.state.isClickable) {
      this.props.updateDataField(this.props.field, alternative);
      this.props.onSubmit();
    }
  }

  render() {
    const alternativeButtons = this.props.alternatives.map(alternative => {
      if (Object.keys(this.props.icons).length > 0) {
        const className = 'btn btn-default' +
          (this.props.value === alternative ? ' selected' : '');
        const color = this.props.value === alternative ? '_white.svg' : '_black.svg';
        return (
          <button
            onClick={() => this.onSelect(alternative)}
            className={className}
            key={alternative}
          >
            <img className="category-image" src={this.props.icons[alternative] + color} alt="ikon" />
            <div className="category-text" >
              {alternative}
            </div>
          </button>
        );
      }
      const className = 'btn btn-default' +
        (this.props.value === alternative ? ' selected' : '');
      return (
        <Button
          label={alternative}
          onClick={() => this.onSelect(alternative)}
          className={className}
          key={alternative}
        />
      );
    });

    return (
      <div className="alternative">
        <div className={`options ${'options-' + alternativeButtons.length}`}>
          {alternativeButtons}
        </div>
      </div>
    );
  }
}

Alternatives.defaultProps = {
  value: ''
};

if (__DEV__) {
  Alternatives.propTypes = {
    alternatives: PropTypes.array.isRequired,
    field: PropTypes.string.isRequired,
    onSubmit: PropTypes.func.isRequired,
    updateDataField: PropTypes.func.isRequired,
    value: PropTypes.string
  };
}

export default Alternatives;
