/* eslint-disable react/forbid-prop-types,react/jsx-boolean-value */
import PropTypes from 'prop-types';
import React, { Component } from 'react';
import ImageViewer from '../../../common/ImageViewer.jsx';
import { openConfirmWindow } from '../../../utilities/warning_window';

class Images extends Component {
  constructor(props) {
    super(props);

    this.state = {
      hideImageViewer: true
    };

    this.onCameraClick = this.onCameraClick.bind(this);
    this.onGalleryClick = this.onGalleryClick.bind(this);
    this.onDeleteImage = this.onDeleteImage.bind(this);
    this.camSuccess = this.camSuccess.bind(this);
    this.camError = this.camError.bind(this);
    this.closeImageViewer = this.closeImageViewer.bind(this);
    this.onImageClick = this.onImageClick.bind(this);
  }

  onCameraClick() {
    const options = {
      destinationType: Camera.DestinationType.FILE_URI,
      encodingType: Camera.EncodingType.JPEG,
      sourceType: Camera.PictureSourceType.CAMERA
    };
    navigator.camera.getPicture(this.camSuccess, this.camError, options);
  }

  onGalleryClick() {
    const options = {
      destinationType: Camera.DestinationType.FILE_URI,
      sourceType: Camera.PictureSourceType.PHOTOLIBRARY
    };
    navigator.camera.getPicture(this.camSuccess, this.camError, options);
  }

  onDeleteImage(index) {
    openConfirmWindow('Er du sikker på at du ønsker å slette bildet?', () => {
      this.props.observationFormActions.deleteFormImage(index);
    });
  }

  camSuccess(imagePath) {
    const that = this;
    function onSuccess(fileEntry) {
      fileEntry.file(file => {
        const reader = new FileReader();
        reader.onload = () => {
          const imageObject = {
            path: imagePath,
            base64: reader.result
          };
          that.props.observationFormActions.addImage(imageObject);
        };
        reader.readAsDataURL(file);
      });
    }

    window.resolveLocalFileSystemURL(
      imagePath,
      onSuccess,
      error => console.log(error)
    );
  }

  camError(error) {
    console.error(error);
  }

  onSubmit() {
    this.props.onSubmit(this.state.count);
  }

  closeImageViewer() {
    this.setState({
      hideImageViewer: true
    });
  }

  onImageClick() {
    this.setState({
      hideImageViewer: false
    });
  }

  render() {
    const formImages = this.props.observationForm.data.images;
    const images = formImages.map((obj, index) => {
      return (
        <div className="thumbnail-container" key={obj.path}>
          <div className="center-helper" />
          <div className="image-thumbnail" onClick={this.onImageClick}>
            <img
              src={obj.path}
              alt="bilde"
              className="image"
            />
          </div>
          <button
            onClick={() => this.onDeleteImage(index)}
            className="delete-button"
          >
            <i className="fa fa-times" />
          </button>
        </div>
      );
    });

    return (
      <div className="images">
        <ImageViewer
          images={formImages}
          close={this.closeImageViewer}
          hide={this.state.hideImageViewer}
          openObservationIds={{}}
          isInForm={true}
        />
        <div className="button-container">
          <button
            onClick={this.onCameraClick}
            className="btn btn-white"
          >
            <i className="fa fa-camera" />
          </button>
          <button
            onClick={this.onGalleryClick}
            className="btn btn-white"
          >
            <i className="fa fa-picture-o" />
          </button>
        </div>
        <div className="image-list">
          {images}
        </div>
      </div>
    );
  }
}

Images.defaultProps = {
  isLastElement: false,
  onSubmit: () => { console.error('No onSubmit prop provided!'); }
};

if (__DEV__) {
  Images.propTypes = {
    onSubmit: PropTypes.func,
    isLastElement: PropTypes.bool
  };
}

export default Images;
