/* eslint-disable no-case-declarations */
import {
  SET_CATEGORY_EXPANDED,
  SET_NEW_OBSERVATION_FORM_CATEGORY,
  SET_NEW_OBSERVATION_FORM_DESCRIPTION,
  ADD_IMAGE,
  DELETE_FORM_IMAGE,
  SET_IMAGES,
  SET_CATEGORY,
  ADD_SHEEP_GROUP,
  INCREMENT_DISPLAY_INDEX,
  INCREMENT_INDEX,
  SET_CURRENT_PANEL,
  SET_DATA,
  SET_DISPLAY_INDEX,
  SET_FORM_STRUCTURE,
  SET_INDEX,
  SUBTRACT_SHEEP_GROUP,
  UPDATE_DATA_FIELD,
  UPDATE_EWES,
  UPDATE_LAMB,
  UPDATE_SHEEP_GROUP,
  RESET_FORM,
  SET_FORM_STATE
} from '../constants/actionTypes/observationFormActionTypes';
import { getSheepGroup } from '../constants/observation/sheep';

const initialState = {
  category: '',
  edit: false,
  index: 0,
  displayIndex: 0,
  data: {},
  status: [],
  formStructure: {}
};

export default function observationForm(state = initialState, action) {
  switch (action.type) {
    case SET_CATEGORY_EXPANDED:
      return {
        ...state,
        categoryExpanded: action.expanded
      };

    case SET_NEW_OBSERVATION_FORM_CATEGORY:
      return {
        ...state,
        selectedCategory: action.category
      };

    case SET_NEW_OBSERVATION_FORM_DESCRIPTION:
      return {
        ...state,
        data: {
          ...state.data,
          description: action.description
        }
      };

    case ADD_IMAGE: {
      const images = state.data.images;
      images.push(action.image);
      return {
        ...state,
        data: {
          ...state.data,
          images
        }
      };
    }

    case DELETE_FORM_IMAGE: {
      const images = state.data.images.filter((image, index) => action.index !== index);
      return {
        ...state,
        data: {
          ...state.data,
          images
        }
      };
    }

    case SET_IMAGES: {
      return {
        ...state,
        images: action.images
      };
    }

    case SET_CATEGORY: {
      return {
        ...state,
        category: action.category
      };
    }

    case SET_CURRENT_PANEL:
      return {
        ...state,
        index: action.index
      };

    case INCREMENT_DISPLAY_INDEX:
      return {
        ...state,
        displayIndex: state.displayIndex + 1
      };

    case SET_DISPLAY_INDEX:
      return {
        ...state,
        displayIndex: action.displayIndex
      };

    case INCREMENT_INDEX:
      return {
        ...state,
        index: state.index + 1
      };

    case SET_INDEX:
      return {
        ...state,
        index: action.index
      };

    case SET_DATA:
      return {
        ...state,
        data: action.data
      };

    case UPDATE_DATA_FIELD: {
      const newState = { ...state };
      newState.data[action.field] = action.value;
      return newState;
    }

    case UPDATE_EWES: {
      const newState = { ...state };
      newState.data.groups[0].ewes = action.ewes;
      return newState;
    }

    case UPDATE_LAMB: {
      const newState = { ...state };
      newState.data.groups[0].lamb = action.lamb;
      return newState;
    }

    case UPDATE_SHEEP_GROUP: {
      const newState = { ...state };
      if (newState.data.groups[action.index] && newState.data.groups[action.index][action.sheepType]) {
        newState.data.groups[action.index][action.sheepType] = action.data;
      }
      return newState;
    }

    case ADD_SHEEP_GROUP: {
      const newState = { ...state };
      const groupElement = {
        ewes: {
          black: 0,
          white: 0,
          blackAndWhite: 0
        },
        lamb: {
          black: 0,
          white: 0,
          blackAndWhite: 0
        }
      };
      newState.data.groups.push(groupElement);

      let formStructure = [...newState.formStructure['Flere grupper']];
      const multipleGroups = newState.formStructure['Flere grupper'];
      formStructure = formStructure.concat(
        getSheepGroup(
          multipleGroups[multipleGroups.length - 1].formInfo.groupIndex + 1
        )
      );
      newState.formStructure['Flere grupper'] = formStructure;

      return newState;
    }

    case SUBTRACT_SHEEP_GROUP: {
      const newState = { ...state };
      // Remove the last data element
      newState.data.groups.pop();
      // Remove the last to panels
      newState.formStructure['Flere grupper'].pop();
      newState.formStructure['Flere grupper'].pop();
      return newState;
    }

    case SET_FORM_STRUCTURE: {
      return {
        ...state,
        formStructure: action.formStructure
      };
    }

    case RESET_FORM: {
      return {
        ...initialState,
        index: -1
      };
    }

    case SET_FORM_STATE: {
      return action.formState;
    }

    default:
      return state;
  }
}
