/* eslint-disable no-case-declarations */
import { objectIsEmpty } from '../utilities/commonHelpMethods.js';
import {
  SAVE_OBSERVATION_ROUND_METADATA,
  ADD_NEW_OBSERVATION,
  ADD_NEW_REGISTRATION_POINT,
  ADD_USER_TRACKING_POINT,
  DELETE_OBSERVATION,
  REGISTER_HOME_POSITION,
  SET_STATE,
  TOGGLE_CHILDREN_VISIBILITY,
  DELETE_IMAGE,
  UPDATE_OBSERVATION_DATA,
  ROUND_HAS_BEEN_UPLOADED,
  FINISH_ROUND
} from '../constants/observationRoundTypes.js';
import { saveState } from '../utilities/save';

const initialState = {
  id: '',
  description: '',
  farmNumber: '',
  buildingNumber: '',
  municipality: '',
  name: '',
  numberOfParticipants: 1,
  weather: '',
  userTrackingPoints: [],
  registrationPoints: {},
  homePosition: {},
  createdTimestamp: '',
  lastSavedTimestamp: '',
  roundIsComplete: false,
  hasBeenUploaded: false
};

export default function observationRound(state = initialState, action) {
  switch (action.type) {

    case REGISTER_HOME_POSITION: {
      return {
        ...state,
        homePosition: action.position
      };
    }

    case ADD_NEW_REGISTRATION_POINT: {
      const id = action.id;
      const registrationPoints = state.registrationPoints;
      registrationPoints[id] = action.registrationPoint;
      registrationPoints[id].timestamp = Date.now();
      const newState = {
        ...state,
        registrationPoints
      };
      saveState(newState);

      return newState;
    }

    case ADD_NEW_OBSERVATION: {
      const parentId = action.parentId;
      const id = action.id;
      const registrationPoints = state.registrationPoints;
      const registrationPoint = registrationPoints[parentId];
      registrationPoint.observations[id] = {
        data: action.data,
        latitude: action.latitude,
        longitude: action.longitude,
        timestamp: Date.now()
      };

      const newState = {
        ...state,
        registrationPoints
      };

      saveState(newState);

      return newState;
    }

    case TOGGLE_CHILDREN_VISIBILITY: {
      const registrationPoints = state.registrationPoints;
      registrationPoints[action.id].childrenIsVisible = !registrationPoints[action.id].childrenIsVisible;
      return {
        ...state,
        registrationPoints
      };
    }

    case DELETE_OBSERVATION: {
      const registrationPoints = state.registrationPoints;
      const observationId = action.observation.observationId;
      const registrationPointId = action.observation.registrationPointId;

      delete registrationPoints[registrationPointId].observations[observationId];
      if (objectIsEmpty(registrationPoints[registrationPointId].observations)) {
        delete registrationPoints[registrationPointId];
      }
      return {
        ...state,
        registrationPoints
      };
    }

    case ADD_USER_TRACKING_POINT: {
      const userTrackingPoints = state.userTrackingPoints;
      userTrackingPoints.push(action.trackingPoint);

      const newState = {
        ...state,
        userTrackingPoints
      };
      saveState(newState);
      return newState;
    }

    case SAVE_OBSERVATION_ROUND_METADATA: {
      const newState = {
        ...action.data,
        userTrackingPoints: [],
        registrationPoints: {},
        homePosition: {},
        roundIsComplete: false,
        hasBeenUploaded: false
      };
      saveState(newState);
      return newState;
    }

    case SET_STATE: {
      return action.state;
    }

    case FINISH_ROUND: {
      const newState = {
        ...state,
        roundIsComplete: true
      };
      saveState(newState);
      return newState;
    }

    case ROUND_HAS_BEEN_UPLOADED: {
      const newState = {
        ...state,
        hasBeenUploaded: true
      };
      saveState(newState);
      return newState;
    }

    case UPDATE_OBSERVATION_DATA: {
      const newState = { ...state };
      newState.registrationPoints[action.registrationPointId].observations[action.observationId].data = action.data;
      saveState(newState);
      return newState;
    }

    case DELETE_IMAGE: {
      const registrationPoints = state.registrationPoints;
      const observation = action.observation;
      const index = action.index;
      if (index > -1) {
        registrationPoints[observation.registrationPointId].observations[observation.observationId].data.images.splice(index, 1);
      }
      const stateToSave = {
        ...state,
        registrationPoints
      };
      saveState(stateToSave);
      return stateToSave;
    }

    default:
      return state;
  }
}
