import axios from 'axios';
import { getFileMapperAsObject, saveState } from './save.js';

const API_BASE_PATH = 'https://4bwp2gdxnc.execute-api.eu-central-1.amazonaws.com/dev/';
const OBSRVATION_ROUNDS = API_BASE_PATH + 'observation-rounds/';
const IMAGES = API_BASE_PATH + 'images/';
const PATH = 'https://4bwp2gdxnc.execute-api.eu-central-1.amazonaws.com/dev/observation-rounds';

/**
 * Returns a promise that converts a file to a base64 string on success.
 *
 * @param file
 * @returns {Promise}
 */
function convertToBase64(file) {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => {
      resolve(reader.result);
    };
    reader.onerror = reject;
  });
}

/**
 * Async function to upload image to server and return S3 path.
 *
 * Takes a paramtert in image which is an image object like:
 *    const file = document.getElementById('photoUpload').files[0];
 * Not sure how these are stored on the phone.
 *
 * Returns a promise. that fires resolve on success, with S3 url as parameter,
 * or fires reject with an error as parameter if an error occurres.
 *
 * Usage:
 *  uploadImage(image)
 *    .then(url => { // handle success })
 *    .catch(error => { // handel error })
 *
 * @param image
 * @returns {Promise}
 */
export function uploadImage(image) {
  return new Promise((resolve, reject) => {
    convertToBase64(image)
      .then(res => {
        const data = JSON.stringify({ image: res });
        axios.post(IMAGES, data)
          .then(response => {
            const imageUrl = response.data.image;
            resolve(imageUrl);
          })
          .catch(error => {
            reject(error);
          });
      })
      .catch(() => {
        reject('Error encoding base64');
      });
  });
}

function stringifyState(state) {
  const data = JSON.stringify(state, (key, value) => {
    if (value === '') {
      return null;
    }
    return value;
  });
  return data;
}

export function uploadRoundData(roundData, success, errorMethod = () => {}) {
  const dataObject = roundData;

  // TODO: Iterate through all images in observations and upload them to API to get S3 urls. The function uploadImage can be sued.

  const data = stringifyState(dataObject);

  axios.post(PATH, data)
    .then(response => {
      console.log(response);
      dataObject.hasBeenUploaded = true;
      saveState(dataObject);
      success();
    })
    .catch(error => {
      console.log(error);
      errorMethod();
    });
}

export function uploadAllRoundData() {
  const data = getFileMapperAsObject();
  Object.keys(data).forEach(key => {
    const observationRound = data[key];
    if (observationRound.roundIsComplete && !observationRound.hasBeenUploaded) {
      uploadRoundData(observationRound, () => {});
    }
  });
}
