
function formatBytes(bytes,decimals) {
  if (bytes === 0) return '0 Bytes';
  const k = 1024,
    dm = decimals || 2,
    sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'],
    i = Math.floor(Math.log(bytes) / Math.log(k));
  return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
}

export function getSizeOfAllTilesAsString(callback) {
  const indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB || window.shimIndexedDB;

  const open = indexedDB.open('leaflet_offline', 2);

  open.onupgradeneeded = () => {
    console.log('upgrade needed');
  };

  open.onsuccess = () => {
    console.log('success');
    const db = open.result;
    const tx = db.transaction('tiles', 'readwrite');
    const store = tx.objectStore('tiles');
    const cursorRequest = store.openCursor();
    let byteCount = 0;
    cursorRequest.onsuccess = event => {
      const result = event.target.result;
      if (result) {
        console.log(formatBytes(result.value.data.length));
        byteCount += result.value.data.length;
        result.continue();
      }
    };

    cursorRequest.onerror = () => {
      console.log('error');
    };

    tx.oncomplete = () => {
      db.close();
      callback(formatBytes(byteCount));
    };
  };
}
