import { initMovingMarkerMoveTime, userIcon } from './mapMarkers.js';
import { openConfirmWindow } from './warning_window';
import { saveFileMapperHasActiveRound } from './save';
import { zoomLevels, deleteTilesButton, saveTilesButton } from '../constants/map';

if (typeof (Number.prototype.toRad) === 'undefined') {
  Number.prototype.toRad = () => {
    return this * Math.PI / 180;
  };
}

if (typeof (Number.prototype.toDeg) === 'undefined') {
  Number.prototype.toDeg = () => {
    return this * (180 / Math.PI);
  };
}

export function getDistance(lat1, lon1, lat2, lon2) {
  const pi = 0.017453292519943295;    // Math.PI / 180
  const cos = Math.cos;
  const result = 0.5 - cos((lat2 - lat1) * pi) / 2 +
    cos(lat1 * pi) * cos(lat2 * pi) *
    (1 - cos((lon2 - lon1) * pi)) / 2;

  return 12742 * Math.asin(Math.sqrt(result)); // 2 * R; R = 6371 km
}

export function setMovingMarkerPosition(map) {
  const initMovingMarkerPosition = [[0, 0], [0, 0]];
  return L.Marker.movingMarker(initMovingMarkerPosition, initMovingMarkerMoveTime, { icon: userIcon }).addTo(map);
}

export function getObservationPosition(map) {
  const latitude = map.getCenter().lat;
  const longitude = map.getCenter().lng;
  return {
    latitude,
    longitude
  };
}

export function checkForActiveRounds(onYes) {
  if (saveFileMapperHasActiveRound()) {
    console.log('File mapper has active round');
    openConfirmWindow(
      'Du har én eller flere aktive runder lagret, ønsker du å laste inn en av disse?',
      () => {
        onYes();
      }
    );
  }
}

export function getDownloadController(onDelete) {
  return {
    zoomlevels: zoomLevels,
    confirm: (layer, successCallback) => {
      openConfirmWindow(
        'Dette området krever at du laster ned ' + layer._tilesforSave.length + ' filer. Ønsker du å laste ned?',
        successCallback,
        () => {
          document.querySelector('.leaflet-draw-edit-edit').click();
        });
    },
    confirmRemoval: (layer, successCallback) => {
      openConfirmWindow(
        'Ønsker du å fjerne alle lagrede kartdata?',
        () => {
          successCallback();
          onDelete();
        },
        () => {
          document.querySelector('.leaflet-draw-edit-edit').click();
        }
      );
    },
    saveText: saveTilesButton,
    rmText: deleteTilesButton
  };
}
