/* eslint-disable no-plusplus */
import { ALTERNATIVES, COUNTER, SHEEP_COLOR, TEXT_FIELD } from '../constants/observation/formElementtTypes';
import { sheepColors } from '../constants/observation/sheepColors';
import sheepTypes from '../constants/observation/sheepTypes';
import { EWE_ALONE, LAMB_ALONE, SHEEP_GROUP, MULTIPLE_GROUPS, FIELD_COLOR } from '../constants/observation/sheep';

function getColorObject(color) {
  return {
    black: color === sheepColors[0] ? 1 : 0,
    white: color === sheepColors[1] ? 1 : 0,
    blackAndWhite: color === sheepColors[2] ? 1 : 0
  };
}

export function getUpdateDataFieldFunction(category, formInfo, formActions, handleChangeCategory) {
  if (category === EWE_ALONE && formInfo.field === FIELD_COLOR) {
    return (fieldName, alternative) => {
      handleChangeCategory(fieldName, alternative);
      formActions.updateEwe(getColorObject(alternative));
    };
  }

  if (category === LAMB_ALONE && formInfo.field === FIELD_COLOR) {
    return (fieldName, alternative) => {
      handleChangeCategory(fieldName, alternative);
      formActions.updateLamb(getColorObject(alternative));
    };
  }

  if (category === SHEEP_GROUP && (formInfo.field === 'ewes' || formInfo.field === 'lamb')) {
    return (index, data) => {
      handleChangeCategory(index, data);
      formActions.updateGroup(index, formInfo.field, data);
    };
  }

  return formActions.updateDataField;
}

export function getHeading(categoryElement, formData) {
  const heading = categoryElement.heading;
  const field = categoryElement.formInfo.field;

  switch (categoryElement.formInfo.type) {
    case ALTERNATIVES:
    case TEXT_FIELD: {
      let data = formData[field];

      // Handle color field of sheep -> "Saøy"/"Lam", these are not directly connected to field, so the data
      //  must be extracted grom the sheep groups object.
      if (categoryElement.formInfo.field === 'color' && (formData.category === EWE_ALONE || formData.category === LAMB_ALONE)) {
        const obj = formData.category === EWE_ALONE ? formData.groups[0].ewes : formData.groups[0].lamb;
        if (obj.black > 0) {
          data = sheepColors[0];
        } else if (obj.white > 0) {
          data = sheepColors[1];
        } else if (obj.blackAndWhite > 0) {
          data = sheepColors[2];
        }
      }

      if (field === 'hasInjury') {  // This field has a question mark, therefore no colon.
        return (typeof data === 'string' && data.length > 0 ? '\u00A0' + data : null);
      }

      return (typeof data === 'string' && data.length > 0 ? ': ' + data : null);
    }
    case COUNTER: {
      const data = formData[field];
      return '' + data;
    }
    case SHEEP_COLOR: {
      if (formData.groups.length > 1) {
        const groupNumber = categoryElement.formInfo.groupIndex + 1;
        return `Gruppe ${groupNumber}: ${categoryElement.heading}`;
      }
      return null;
    }
    default:
      return null;
  }
}

export function getNumberOfPanels(observationForm) {
  if (observationForm.category === 'Sau') {
    const subcategory = observationForm.data.category;
    if (subcategory === SHEEP_GROUP) {
      const extra = observationForm.data.injuries ? (observationForm.data.injuries.length * 5) : 0;
      return Object.keys(observationForm.data).length + extra - 1;
    } else if (subcategory === MULTIPLE_GROUPS) {
      return Object.keys(observationForm.data).length + ((2 * observationForm.data.groups.length) - 2);
    }
    return Object.keys(observationForm.data).length - 2;
  } else if (observationForm.category === 'Hund') {
    const {category} = observationForm.data;
    if (category === 'Hund alene' || category === 'Hund med eier') {
      return Object.keys(observationForm.data).length - 2;
    }
  } else if (observationForm.category === 'Skadet sau') {
    const { category } = observationForm.data;
    if (category === 'Annet') {
      return 3;
    }
  } else if (observationForm.category === 'Rovdyr') {
    return Object.keys(observationForm.data).length - 1;
  } else if (observationForm.category === 'Ullfunn') {
    return Object.keys(observationForm.data).length - 1;
  } else if (observationForm.category === 'Død sau') {
    return Object.keys(observationForm.data).length - 1;
  } else if (observationForm.category === 'Reinsdyr') {
    return Object.keys(observationForm.data).length - 1;
  } else if (observationForm.category === 'Annet') {
    return Object.keys(observationForm.data).length - 1;
  }
  return Object.keys(observationForm.data).length - 2;
}

export function getCategory(category) {
  switch (category) {
    case 'dog':
    case 'Hund':
    case 'Hund alene':
    case 'Hund med eier': {
      return 'Hund';
    }
    case 'other':
      return 'Annet';
    case 'injured sheep':
    case 'Halter':
    case 'Hodeskade':
    case 'Kroppskade':
    case 'Blødning':
    case 'Skadet sau':
    case 'Annet': {
      return 'Skadet sau';
    }
    case 'Reinsdyr':
    case 'reindeer':
      return 'Reinsdyr';
    case 'predator':
    case 'Rovdyr':
    case 'Rovfugl': {
      return 'Rovdyr';
    }
    case 'sheep':
    case EWE_ALONE:
    case LAMB_ALONE:
    case SHEEP_GROUP:
    case 'Sau':
    case MULTIPLE_GROUPS: {
      return 'Sau';
    }
    case 'deadSheep':
    case 'Død sau':
      return 'Død sau';
    case 'wool':
    case 'Ullfunn':
      return 'Ullfunn';
    default:
      return '';
  }
}

function getKeysWithHierarchyLevel(elements, level = 0, parent = null) {
  let hasChildren = false;
  console.log('elements', elements)
  for (let i = 0; i < elements.length; i++) {
    if (elements[i].subcategories !== undefined) {
      hasChildren = true;
    }
  }

  if (!hasChildren) {
    return elements;
  }

  let keys = [];

  for (let i = 0; i < elements.length; i++) {
    keys.push({
      field: elements[i].formInfo.field,
      level,
      parent
    });
    if (elements[i].subcategories !== undefined) {
      const categoryKeys = Object.keys(elements[i].subcategories);

      for (let j = 0; j < categoryKeys.length; j++) {
        const sub = elements[i].subcategories[categoryKeys[j]];
        if (sub.subcategories !== undefined) {
          keys = keys.concat(getKeysWithHierarchyLevel([sub], level + 1, elements[i].formInfo.field));
        } else {
          if (sub instanceof Array) {
            const l = [];
            for (let k = 0; k < sub.length; k++) {
              l.push({
                field: sub[k].formInfo.field,
                level: (level + 1),
                parent: elements[i].formInfo.field
              });
            }
            keys = keys.concat(l);
          } else {
            keys.push({
              field: sub.formInfo.field,
              level: (level + 1),
              parent: elements[i].formInfo.field
            });
          }
        }
      }
    }
  }

  return keys;
}

function getParentLIst(elements, field, initField = field, parents = [], n = 0) {


  if (n > 500) {
    console.error('Prbably overflow');
    console.log(elements)
    return;
  }
  for (let i = 0; i < elements.length; i++) {
    const element = elements[i];
    if (element.field === field) {
      if (element.parent === null || element.parent === undefined) {
        return {
          field: initField,
          parents
        };
      } else {
        const p = parents.concat([element.parent]);
        return getParentLIst(elements, element.parent, initField, p, (n + 1));
      }
    }
  }
  return null;
}

function getParentLIstAll(elements) {
  const parents = [];
  for (let i = 0; i < elements.length; i++) {
    parents.push(getParentLIst(elements, elements[i].field))
  }
  return parents;
}

export function arrayContains(l, e) {
  return l.indexOf(e) >= 0;
}

export function getKeysToDelete(formDataStructure, field) {
  const { elements } = formDataStructure;
  const keyLevelList = elements !== undefined ? getKeysWithHierarchyLevel(elements) : getKeysWithHierarchyLevel(formDataStructure);

  const a = getParentLIstAll(keyLevelList, field);
  const res = [];
  for (let i = 0; i < a.length; i++) {
    if (a[i].parents.indexOf(field) >= 0 && !arrayContains(res, a[i].field)) {
      res.push(a[i].field);
    }
  }
  return res;
}

function removeNumbersFromString(s) {
  return s.replace(/[0-9]/g, '');
}

function removeCharactersFromString(s) {
  const res = s.replace(/\D/g, '');
  return res === '' ? 0 : parseInt(res, 10);
}

function removeInjuryFields(data) {
  const res = { ...data };
  const keys = Object.keys(data);

  for (let i = 0; i < keys.length; i++) {
    switch (removeNumbersFromString(keys[i])) {
      case 'injuryCategory':
      case 'injury':
      case 'injuryColor':
      case 'sheepType':
      case 'decision':
        delete res[keys[i]];
        break;
      default:
        break;
    }
  }

  return res;
}

const injuryObject = {
  injuryCategory: '',
  injury: '',
  sheepType: '',
  color: '',
  decision: ''
};

function getEmptyInjuriesList(data) {
  const injuries = [];
  for (let i = 0; i < data.count; i++) {
    injuries.push({ ...injuryObject });
  }
  return injuries;
}

function isGroupAndDefaultKey(data, key) {
  const isSheepGroup = data.category === SHEEP_GROUP;
  const isDefaultKey = removeNumbersFromString(key) === key;
  return isSheepGroup && isDefaultKey;
}

export function getFormattedObservationData(data) {
  const keys = Object.keys(data);

  if (data.hasInjury === 'Ja') {
    const injuries = getEmptyInjuriesList(data);

    for (let i = 0; i < keys.length; i++) {
      if (!isGroupAndDefaultKey(data, keys[i])) {
        const injuriesElement = injuries[removeCharactersFromString(keys[i])];
        switch (removeNumbersFromString(keys[i])) {
          case 'injuryCategory':
            injuriesElement.injuryCategory = data[keys[i]];
            break;
          case 'injury':
            injuriesElement.injury = data[keys[i]];
            break;
          case 'injuryColor':
            injuriesElement.color = data[keys[i]];
            break;
          case 'decision':
            injuriesElement.decision = data[keys[i]];
            break;
          case 'sheepType':
            injuriesElement.sheepType = data[keys[i]];
            break;
          default:
            break;
        }
      }
    }

    // Set sheepType 'Søye' if category ewe alone is selected.
    if (data.category === EWE_ALONE && injuries.length === 1) {
      injuries[0].sheepType = sheepTypes[0];
    }


    // Set sheepType 'Lam' if category lamb alone is selected.
    if (data.category === LAMB_ALONE && injuries.length === 1) {
      injuries[0].sheepType = sheepTypes[1];
    }

    const res = { ...data };
    res.injuries = injuries;
    return res;
  }

  return data;
}

