
import { isNullOrUndefined, objectIsEmpty } from './commonHelpMethods';

const saveFileMapper = 'saveFileMapper';
const savedAreas = 'savedAreas';
const settings = 'settings';

export function getFileMapperAsObject() {
  return JSON.parse(localStorage.getItem(saveFileMapper));
}

export function getSavedAreasAsList() {
  const list = JSON.parse(localStorage.getItem(savedAreas));
  return isNullOrUndefined(list) ? [] : list;
}

export function createSaveFileMapper() {
  const object = {};
  localStorage.setItem(saveFileMapper, JSON.stringify(object));
}

export function saveFileMapperIsEmpty() {
  const fileMapper = getFileMapperAsObject();
  return fileMapper === null;
}

function savedAreasIsEmpty() {
  const savedAreasAsList = getSavedAreasAsList();
  return savedAreasAsList === null;
}

export function saveFileMapperHasActiveRound() {
  if (saveFileMapperIsEmpty()) {
    return false;
  }
  const fileMapper = getFileMapperAsObject();
  let result = false;
  Object.keys(fileMapper).forEach(key => {
    const observationRound = fileMapper[key];
    if (!observationRound.roundIsComplete) {
      result = true;
    }
  });

  return result;
}

export function checkSaveFileMapper() {
  if (saveFileMapperIsEmpty()) {
    createSaveFileMapper();
  }
}

export function saveState(state) {
  if (saveFileMapperIsEmpty()) {
    createSaveFileMapper();
  }
  const file = state;
  const fileMapper = getFileMapperAsObject();
  file.lastSavedTimestamp = Date.now();
  fileMapper[file.id] = file;
  localStorage.setItem(saveFileMapper, JSON.stringify(fileMapper));
}

export function deleteObservationRoundById(id) {
  const fileMapper = getFileMapperAsObject();
  delete fileMapper[id];
  localStorage.setItem(saveFileMapper, JSON.stringify(fileMapper));
}

export function getNewestOngoingObservationRound() {
  const fileMapper = getFileMapperAsObject();

  let newestActiveObservationRound = {};

  Object.keys(fileMapper).forEach(key => {
    const observationRound = fileMapper[key];
    if (!observationRound.roundIsComplete) {
      if (objectIsEmpty(newestActiveObservationRound)) {
        newestActiveObservationRound = observationRound;
      } else {
        if (observationRound.lastSavedTimestamp > newestActiveObservationRound.lastSavedTimestamp) {
          newestActiveObservationRound = observationRound;
        }
      }
    }
  });

  return newestActiveObservationRound;
}

export function saveArea(coordinates) {
  if (savedAreasIsEmpty()) {
    const list = [];
    localStorage.setItem(savedAreas, JSON.stringify(list));
  }
  const savedAreasAsList = getSavedAreasAsList();
  savedAreasAsList.push(coordinates);
  localStorage.setItem(savedAreas, JSON.stringify(savedAreasAsList));
}

export function clearSavedAreas() {
  localStorage.setItem(savedAreas, JSON.stringify([]));
}

export function saveSettings(newSettings) {
  localStorage.setItem(settings, JSON.stringify(newSettings));
}

export function getSettings() {
  let storedSettings = JSON.parse(localStorage.getItem(settings));
  if (storedSettings === null || storedSettings === undefined) {
    storedSettings = {
      userTrackingIsOn: true,
      automaticUploadIsEnabled: true
    };
    saveSettings(storedSettings);
  }
  return storedSettings;
}
