import Velocity from 'velocity-animate';

const WIDTH = document.documentElement.clientWidth;
const SWIPE_THRESHOLD = 30;
const ABSOLUTE_LEFT = -WIDTH * 0.85;
const ABSOLUTE_RIGHT = 0;
const scrollspeed = 5;
const MAX_SWIPE_TIME = 0.25;
let swipestart;
let swipeend;
let mousePosition;
let offset = 0;
let isDown = false;
let xDown = null;
let div;

let onMenuClose;

if (document.createEvent) {
  onMenuClose = document.createEvent('HTMLEvents');
  onMenuClose.initEvent('onMenuClose', true, true);
}
onMenuClose.eventName = 'onMenuClose';


function getSwipeDirection(down, up) {
  const difference = down - up;
  let output = '';
  if (difference > SWIPE_THRESHOLD) {
    output = 'left';
  } else if (difference < -SWIPE_THRESHOLD) {
    output = 'right';
  } else {
    output = 'no swipe';
  }
  return output;
}

function animateClose() {
  Velocity(div, { left: ABSOLUTE_LEFT }, { duration: 250, complete: () => div.dispatchEvent(onMenuClose) });
}

function animateOpen() {
  Velocity(div, { left: ABSOLUTE_RIGHT }, { duration: 250 });
}

function onTouchStart(e) {
  swipestart = new Date().getTime() / 1000;
  isDown = true;
  offset = div.offsetLeft - e.touches[0].clientX;
  xDown = e.touches[0].clientX;
}

function onTouchEnd(e) {
  swipeend = new Date().getTime() / 1000;
  isDown = false;
  mousePosition = e.changedTouches[0].clientX;
  const direction = getSwipeDirection(xDown, e.changedTouches[0].clientX);
  const swipetime = swipeend - swipestart;
  if (direction !== 'no swipe' && swipetime < MAX_SWIPE_TIME) {
    if (direction === 'left') {
      animateClose();
    } else {
      animateOpen();
    }
  } else if ((mousePosition + offset) < (ABSOLUTE_LEFT / 2)) {
    animateClose();
  } else {
    animateOpen();
  }
}

function onTouchMove(e) {
  e.preventDefault();
  if (isDown) {
    mousePosition = e.touches[0].clientX;
    if ((mousePosition + offset) < 0 && (mousePosition + offset) > ABSOLUTE_LEFT) {
      Velocity(div, { left: mousePosition + offset }, { duration: 0 });
    }
  }
}

export function init(id, closeFunction) {
  div = document.getElementById(id);

  div.addEventListener('onMenuClose', closeFunction, true);
  div.addEventListener('touchstart', onTouchStart, true);
  div.addEventListener('touchend', onTouchEnd, true);
  div.addEventListener('touchmove', onTouchMove, true);
}

export function removeEventListeners(closeFunction) {
  div.removeEventListener('onMenuClose', closeFunction, true);
  div.removeEventListener('touchstart', onTouchStart, true);
  div.removeEventListener('touchend', onTouchEnd, true);
  div.removeEventListener('touchmove', onTouchMove, true);
}