import React, {Component} from 'react';
import {connect} from 'react-redux';
import {bindActionCreators} from 'redux';
import {getDateAsString} from '../utilities/dateParser';
import * as observationRoundMapActions from '../actions/observationRoundMapActions';


class List extends Component {

  constructor(props) {
    super(props);
    this.state = {
      toggleAllIsChecked: false,
      sortBy: 'farmNumber',
      activeSortDirection: 1
    };
    this.onToggleAll = this.onToggleAll.bind(this);
    this.sortBy = this.sortBy.bind(this);
    this.compare = this.compare.bind(this);
  }


  onItemToggle(event, id) {
    if (event.target.checked) {
      this.props.observationRoundMapActions.addIds([id]);
    } else {
      this.props.observationRoundMapActions.removeIds([id]);
      this.setState({
        toggleAllIsChecked: false
      });
    }
  }

  onToggleAll(event) {
    const allRoundIds = [];
    this.props.observationRounds.forEach(round => allRoundIds.push(round.id));
    if (event.target.checked) {
      this.props.observationRoundMapActions.addIds(allRoundIds);
      this.setState({
        toggleAllIsChecked: true
      });
    } else {
      this.props.observationRoundMapActions.removeIds(allRoundIds);
      this.setState({
        toggleAllIsChecked: false
      });
    }
  }

  sortBy(index) {
    let activeSortDirection = 1;
    if (this.props.chosenData[index] !== this.state.sortBy) {
      activeSortDirection = 1;
    } else {
      if (this.state.activeSortDirection === -1) {
        activeSortDirection = 1;
      } else if(this.state.activeSortDirection === 1) {
        activeSortDirection = -1;
      }
    }
    this.setState({
      sortBy: this.props.chosenData[index],
      activeSortDirection
    })
  }

  compare(a, b) {
    if (a[this.state.sortBy] < b[this.state.sortBy])
      return -1 * this.state.activeSortDirection;
    if (a[this.state.sortBy] > b[this.state.sortBy])
      return this.state.activeSortDirection;
    return 0;
  }

  render() {
    const loading = this.props.loading;
    const error = this.props.error;
    const activeIds = this.props.observationRoundMap.ids;
    const activeSort = this.state.sortBy;

    const headerItems = this.props.headers.map((header, index) => {
      let icon = 'fa-arrows-alt-v';
      if (activeSort === this.props.chosenData[index]) {
        if (this.state.activeSortDirection === 1) {
          icon = 'fa-chevron-down';
        } else if (this.state.activeSortDirection === -1) {
          icon = 'fa-chevron-up';
        }
      }
      return (
        <div className="list-header-item" key={header} onClick={() => this.sortBy(index)}>
          <div className="header-text">
            {header}
          </div>
          <i className={'fa ' + icon}/>
        </div>
      );
    });
    const rounds = this.props.observationRounds;
    const sortedItems = rounds.sort(this.compare);
    const listItems = sortedItems.map(round => {
      return (
        <div className="list-row" key={round.id}>
          <div className="list-item">
            <input
              type="checkbox"
              checked={activeIds.includes(round.id)}
              onChange={e => this.onItemToggle(e, round.id)}
            />
          </div>
          {this.props.chosenData.map(key => {
            const value = key === 'createdTimestamp' ? getDateAsString(round[key]) : round[key];

            return (
              <div className="list-item" key={round.id + key}>
                {value}
              </div>
            );
          })}
        </div>
      )
    });

    let content = '';

    if (loading) {
      content = (
        <div className="spinner-container">
          <i className="fa fa-spinner fa-pulse fa-3x fa-fw" />
        </div>
      );
    } else if(error) {
      content = <div>Kunne ikke laste data</div>;
    } else {
      content = listItems;
    }

    const toggleAllIsChecked = this.props.observationRoundMap.ids.length === this.props.observationRounds.length ? true : this.state.toggleAllIsChecked;

    return (
      <div className="list-container">
        <div className="list-header">
          <div className="list-header-item">
            <input
              type="checkbox"
              onChange={this.onToggleAll}
              checked={toggleAllIsChecked}
            />
          </div>
          {headerItems}
        </div>
        <div className="list-body-container">
          <div className="list-body">
            {content}
          </div>
        </div>
      </div>
    )
  }
}

function stateToProps(state) {
  return {
    observationRoundMap: state.observationRoundMap
  };
}

function dispatchToProps(dispatch) {
  return {
    observationRoundMapActions: bindActionCreators(observationRoundMapActions, dispatch)
  };
}

export default connect(stateToProps, dispatchToProps)(List);