import React, { Component } from 'react';
import {connect} from 'react-redux';
import {bindActionCreators} from 'redux';
import * as observationRoundMapActions from '../actions/observationRoundMapActions';
import {
  getStateByObservation,
  getIcon,
  drawRegistrationPointsAndObservations,
  getMarker,
  mapStyle,
  mapUrl
} from '../utilities/mapHelper';
import {
  Map,
  Polyline,
  TileLayer } from 'react-leaflet'

class MapComponent extends Component {

  constructor(props) {
    super(props);
    const observationRounds = [];
    this.state = {
      lat: 63.426739,
      lon: 10.382404,
      observationRounds
    };
  };

  componentWillReceiveProps(props) {
    const ids = props.observationRoundMap.ids;
    const allRounds = props.observationRounds;
    const observationRounds = [];
    allRounds.forEach(round => {
      if (ids.includes(round.id)) {
        observationRounds.push(round);
      }
    });
    this.setState({
      observationRounds
    });
  }

  render() {
    return (
      <Map center={[this.state.lat, this.state.lon]} zoom={14} style={mapStyle}>
        <TileLayer url={mapUrl}/>
        {this.state.observationRounds.map(round => {
          return drawRegistrationPointsAndObservations(
              this.props.observationRoundMap,
              round,
              this.props.displayType
          );
        })}
      </Map>
    );
  }
}

function stateToProps(state) {
  return {
    observationRoundMap: state.observationRoundMap
  };
}

function dispatchToProps(dispatch) {
  return {
    observationRoundMapActions: bindActionCreators(observationRoundMapActions, dispatch)
  };
}

export default connect(stateToProps, dispatchToProps)(MapComponent);
