import React, { Component } from 'react';
import {connect} from 'react-redux';
import {bindActionCreators} from 'redux';
import * as observationRoundMapActions from '../actions/observationRoundMapActions';
import {
  drawRegistrationPointsAndObservations,
  mapStyle,
  mapUrl
} from '../utilities/mapHelper';
import {
  Map,
  TileLayer
} from 'react-leaflet'

class SingleObservationMap extends Component {

  constructor(props) {
    super(props);
    this.state = {
      lat: 63.426739,
      lon: 10.382404,
      observationRound: this.props.observationRound
    };
  };

  componentWillReceiveProps(props) {
    this.setState({
      observationRound: props.observationRound
    });
  }


  render() {
    const observationRound = this.state.observationRound;
    const homePosition = [observationRound.homePosition.latitude, observationRound.homePosition.longitude];

    return (
        <Map center={homePosition} zoom={14} style={this.props.style} bounds={this.props.bounds} boundsOptions={{padding: [20, 30]}} zoomControl={false}>
          <TileLayer url={mapUrl}/>
          {drawRegistrationPointsAndObservations(this.props.observationRoundMap, this.state.observationRound)}
        </Map>
    );
  }
}

function stateToProps(state) {
  return {
    observationRoundMap: state.observationRoundMap
  };
}

function dispatchToProps(dispatch) {
  return {
    observationRoundMapActions: bindActionCreators(observationRoundMapActions, dispatch)
  };
}

export default connect(stateToProps, dispatchToProps)(SingleObservationMap);
