import React, { Component } from 'react';
import {
  getStateByObservation,
  getIcon,
  drawRegistrationPointsAndObservations,
  getMarker,
  mapStyle,
  mapUrl
} from '../utilities/mapHelper';
import {
  Map,
  Polyline,
  TileLayer } from 'react-leaflet'

class TimeLapseMap extends Component {

  constructor(props) {
    super(props);

    let state = getStateByObservation(this.props.observationRound);
    this.state = {
      lat: 63.426739,
      lon: 10.382404,
      ...state,
      showUserTrackingLine: this.props.showUserTrackingLine,
      showRegistrationPoints: true,
      showObservationPoints: true
    };
    this.getGraphics = this.getGraphics.bind(this);
    this.getUserPolyline = this.getUserPolyline.bind(this);
  };

  componentWillReceiveProps(props) {
    let state = getStateByObservation(props.observationRound);
    this.setState({
        ...state,
        showUserTrackingLine: props.showUserTrackingLine
    })
  }

  getUserPolyline() {
    if (this.state.showUserTrackingLine) {
      return <Polyline positions={this.state.userPolyline} color="lime"/>
    }
    return null;
  }

  getGraphics() {
    return drawRegistrationPointsAndObservations(
      this.state.registrationPoints,
      this.state.showRegistrationPoints,
      this.state.showObservationPoints
    )
  }

  render() {
    return (
      <Map center={this.state.homePosition} zoom={14} style={mapStyle}>
        <TileLayer url={mapUrl}/>
        {getMarker(this.state.homePosition, 'home')}
        {this.getUserPolyline()}
        {this.getGraphics()}
      </Map>
    );
  }
}

export default TimeLapseMap;
