import React, {Component} from 'react';
import {connect} from 'react-redux';
import {bindActionCreators} from 'redux';
import * as observationRoundActions from '../actions/observationRoundActions';
import * as authActions from '../actions/authActions';
import * as observationRoundMapActions from '../actions/observationRoundMapActions';
import List from '../common/List';
import MapComponent from '../common/MapComponent';
import '../style/App.css';
import '../style/AppStyle.css';
import '../style/fontawesome/css/fontawesome-all.css';
import Auth from './../auth/Auth';

const auth = new Auth();
const handleAuthentication = (nextState, historyPush) => {
  if (/access_token|id_token|error/.test(nextState.location.hash)) {
    auth.handleAuthentication(historyPush);
  }
};

const headers = ['Gårdsnummer', 'Navn', 'Dato', 'Kommune'];
const chosenData = ['farmNumber', 'name', 'createdTimestamp', 'municipality'];
const mapStyle = {
  height: '500px',
  width: '100%'
};


class Map extends Component {

  constructor(props) {
    super(props);

    this.onDisplayChange = this.onDisplayChange.bind(this);
  }

  componentDidMount() {
    this.props.observationRoundActions.fetchObservationRounds();
  }

  toggleFilter(event, filter) {
    const checked = event.target.checked;
    switch(filter) {
      case 'userTrackingLine': {
        this.props.observationRoundMapActions.toggleUserTrackingLine(checked);
        break;
      }
      case 'registrationPoints': {
        this.props.observationRoundMapActions.toggleRegistrationPoints(checked);
        break;
      }
      case 'observationPoints': {
        this.props.observationRoundMapActions.toggleObservationPoints(checked);
        break;
      }
      case 'sheep': {
        this.props.observationRoundMapActions.toggleSheep(checked);
        break;
      }
      case 'deadSheep': {
        this.props.observationRoundMapActions.toggleDeadSheep(checked);
        break;
      }
      case 'woolFindings': {
        this.props.observationRoundMapActions.toggleWoolFindings(checked);
        break;
      }
      case 'dogs': {
        this.props.observationRoundMapActions.toggleDogs(checked);
        break;
      }
      case 'predators': {
        this.props.observationRoundMapActions.togglePredators(checked);
        break;
      }
      case 'other': {
        this.props.observationRoundMapActions.toggleOther(checked);
        break;
      }
      default:
        console.log('asd');
    }
  }

  onDisplayChange(event) {
    this.props.observationRoundMapActions.changeDisplay(event.target.value);
  }

  standardFilterOptions() {
    return (
        <div className="filters-container">
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showUserTrackingLine}
                onChange={event => this.toggleFilter(event, 'userTrackingLine')}
            />
            <div className="filter-text">
              Gålinje
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showRegistrationPoints}
                onChange={event => this.toggleFilter(event, 'registrationPoints')}
            />
            <div className="filter-text">
              Registreringspunkter
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showObservationPoints}
                onChange={event => this.toggleFilter(event, 'observationPoints')}
            />
            <div className="filter-text">
              Observasjoner
            </div>
          </div>
        </div>
    )
  }

  intensityFilterOptions() {
    return (
        <div className="filters-container">
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showSheep}
                onChange={event => this.toggleFilter(event, 'sheep')}
            />
            <div className="filter-text">
              Sau
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showDeadSheep}
                onChange={event => this.toggleFilter(event, 'deadSheep')}
            />
            <div className="filter-text">
              Død sau
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showWoolFindings}
                onChange={event => this.toggleFilter(event, 'woolFindings')}
            />
            <div className="filter-text">
              Ullfunn
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showDogs}
                onChange={event => this.toggleFilter(event, 'dogs')}
            />
            <div className="filter-text">
              Hund
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showPredators}
                onChange={event => this.toggleFilter(event, 'predators')}
            />
            <div className="filter-text">
              Rovdyr
            </div>
          </div>
          <div className="filter">
            <input
                className="checkbox"
                type="checkbox"
                checked={this.props.observationRoundMap.showOther}
                onChange={event => this.toggleFilter(event, 'other')}
            />
            <div className="filter-text">
              Annet
            </div>
          </div>
        </div>
    )
  }

  getFilter() {
    switch (this.props.observationRoundMap.display){
      case 'standard': {
        return this.standardFilterOptions();
      }
      case 'intensity': {
        return this.intensityFilterOptions();
      }
      default:
        return this.standardFilterOptions();
    }
  }

  render() {
    const loading = this.props.observationRounds.status.fetching;
    const error = this.props.observationRounds.status.error;
    return (
      <div className="map-content">
        <div className="filter-container">
          <div className="display-picker">
            <div className="filter-text">
              Visning:
            </div>
            <select value={this.props.observationRoundMap.display} onChange={this.onDisplayChange} className="select">
              <option value="standard">Standard</option>
              <option value="intensity">Intensitet</option>
            </select>
          </div>
          {this.getFilter()}
        </div>
        <div className="map-container">
          <MapComponent displayType={this.props.observationRoundMap.display} observationRounds={this.props.observationRounds.data} />
        </div>
        <List
          loading={loading}
          error={error}
          headers={headers}
          chosenData={chosenData}
          observationRounds={this.props.observationRounds.data}
        />
      </div>
    );
  }
}

function stateToProps(state) {
  return {
    auth: state.auth,
    observationRounds: state.observationRounds,
    observationRoundMap: state.observationRoundMap
  };
}

function dispatchToProps(dispatch) {
  return {
    authActions: bindActionCreators(authActions, dispatch),
    observationRoundActions: bindActionCreators(observationRoundActions, dispatch),
    observationRoundMapActions: bindActionCreators(observationRoundMapActions, dispatch)
  };
}

export default connect(stateToProps, dispatchToProps)(Map);