import React, {Component} from 'react';
import {connect} from 'react-redux';
import {bindActionCreators} from 'redux';
import { latLngBounds } from 'leaflet';
import {
  compare,
  saveDocumentAsPdf,
  observationRoundBasicInfo,
  observationRoundDetailedInfo
} from '../utilities/reportHelper';
import * as observationRoundActions from '../actions/observationRoundActions';
import * as testActions from '../actions/testActions';
import Navbar from './ReportNavbar';


class ObservationRounds extends Component {
  constructor(props) {
    super(props);

    this.state = {
      showDetailedReport: true
    };
    this.getInfoForAllRounds = this.getInfoForAllRounds.bind(this);
    this.toggleFilter = this.toggleFilter.bind(this);
  }

  componentDidMount() {
    this.props.observationRoundActions.fetchObservationRounds();
    console.log(this.props.observationRounds);
  }

  getInfoForAllRounds() {
    let output = [];
    const rounds = this.props.observationRounds.data;
    const sortedRounds = rounds.sort(compare);

    sortedRounds.forEach((round, index) => {
      output.push(observationRoundBasicInfo(round, index));
      if (this.state.showDetailedReport) {
        output.push(<div key={'header' + index} className="header-1">Observasjoner</div>);
        output = [...output, observationRoundDetailedInfo(round)];
      }
    });

    return output;
  }

  toggleFilter(event) {
    this.setState({
      showDetailedReport: event.target.checked
    });
  }


  render() {
    return (
        <div className="report-container">
          <Navbar checked={this.state.showDetailedReport} onChange={this.toggleFilter} onSave={saveDocumentAsPdf} />
          <div id="report-pages" className="report-pages">
            <div className="report-page">
              <div className="report-overview">
                <div className="title">
                  Sesongrapport for tilsynsturer
                </div>
                <div className="text">
                  Introduksjonstekst til selve rapporten + statistikk, typ antall turer, gjennomsnittstid mellom hver tur, totalt antall døde sauer, total antall skadde sauer, antall rovdyr observert osv?
                  Kan også ha med forklaring rundt tegnsetting, typ <i className="fa fa-home" /> betyr start osv.

                  Bug: virker ikke som om description fungerer. Blir bare satt til null.
                </div>
              </div>
              <div className="observation-rounds">
                {this.getInfoForAllRounds()}
              </div>
            </div>
          </div>
        </div>
    )
  }
}

function stateToProps(state) {
  return {observationRounds: state.observationRounds};
}

function dispatchToProps(dispatch) {
  return {
    observationRoundActions: bindActionCreators(observationRoundActions, dispatch),
    testActions: bindActionCreators(testActions, dispatch)
  };
}

export default connect(stateToProps, dispatchToProps)(ObservationRounds);