import * as types from '../constants/actionTypes/observationRoundMapActionTypes';


const initialState = {
  ids: [],
  showUserTrackingLine: true,
  showRegistrationPoints: true,
  showObservationPoints: true,
  showSheep: true,
  showDeadSheep: true,
  showWoolFindings: true,
  showDogs: true,
  showPredators: true,
  showOther: true,
  display: 'standard'
};

export default (state = initialState, action) => {
  switch (action.type) {
    case types.ADD_IDS_TO_LIST:
      return {
        ...state,
        ids: [...state.ids, ...action.ids]
      };

    case types.REMOVE_IDS_FROM_LIST:
      const currentIds = state.ids;
      const idsToRemove = action.ids;
      const newIds = currentIds.filter(id => !idsToRemove.includes(id));

      return {
        ...state,
        ids: newIds
      };

    case types.TOGGLE_USERLINE: {
      return {
        ...state,
        showUserTrackingLine: action.showUserTrackingLine
      }
    }

    case types.TOGGLE_REGISTRATION_POINTS: {
      return {
        ...state,
        showRegistrationPoints: action.showRegistrationPoints
      }
    }

    case types.TOGGLE_OBSERVATIONS: {
      return {
        ...state,
        showObservationPoints: action.showObservationPoints
      }
    }

    case types.TOGGLE_SHEEP: {
      return {
        ...state,
        showSheep: action.showSheep
      }
    }

    case types.TOGGLE_DEAD_SHEEP: {
      return {
        ...state,
        showDeadSheep: action.showDeadSheep
      }
    }

    case types.TOGGLE_WOOL_FINDINGS: {
      return {
        ...state,
        showWoolFindings: action.showWoolFindings
      }
    }

    case types.TOGGLE_DOGS: {
      return {
        ...state,
        showDogs: action.showDogs
      }
    }

    case types.TOGGLE_PREDATORS: {
      return {
        ...state,
        showPredators: action.showPredators
      }
    }

    case types.TOGGLE_OTHER: {
      return {
        ...state,
        showOther: action.showOther
      }
    }

    case types.CHANGE_DISPLAY: {
      if (action.display === 'standard') {
        return {
          ...state,
          showSheep: true,
          showDeadSheep: true,
          showWoolFindings: true,
          showDogs: true,
          showPredators: true,
          showOther: true,
          display: action.display
        }
      }else {
        return {
          ...state,
          showUserTrackingLine: true,
          showRegistrationPoints: true,
          showObservationPoints: true,
          display: action.display
        }
      }
    }

    default:
      return state
  }
};
