import L from 'leaflet';
import { getNumberOfSheepInObservation } from './statistics.js';

import sheepImage from '../style/img/vectors/sheep-small-icon.svg';
import dogImage from '../style/img/vectors/dog-icon-small.svg';
import reindeerImage from '../style/img/vectors/reindeer-icon-small.svg';
import predatorImage from '../style/img/vectors/bear-icon-small.svg';
const baseUrl = '../style/img/vectors/';
const markerIcons = {
  'Sau': {
    url: sheepImage,
    className: 'sheep-observation-marker'
  },
  'Skadet sau': {
    url: sheepImage,
    className: 'sheep-observation-marker'
  },
  'Død sau': {
    url: sheepImage,
    className: 'sheep-observation-marker'
  },
  'Ullfunn': {
    url: sheepImage,
    className: 'sheep-observation-marker'
  },
  'Hund': {
    url: dogImage,
    className: 'sheep-observation-marker'
  },
  'Rovdyr': {
    url: predatorImage,
    className: 'sheep-observation-marker'
  },
  'Reinsdyr': {
    url: reindeerImage,
    className: 'sheep-observation-marker'
  },
  'Annet': {
    url: `${baseUrl}sheep-small-icon.svg`,
    className: 'sheep-observation-marker'
  }
};

export const observationPolyLineStyle = {
  color: 'blue',
  weight: 3,
  opacity: 1,
  smoothFactor: 1
};

const markerSettings = {
  iconSize: [33, 56],
  iconAnchor: [16.5, 56],
  popupAnchor: [-3, -76]
};

export const userIcon = L.divIcon({ className: 'user-icon' });
export const initMovingMarkerMoveTime = [1000];



function getImageIcon(data) {
  if (data.images.length > 0) {
    return '<div class="fa fa-picture-o observation-marker-image"></div>';
  }
  return '';
}

function getUtilityMarker(className) {
  return new L.divIcon({
    html: '<div class="marker-container">' +
    '<div class="marker-icon">' +
    '<i class="' + className + '"></i>' +
    '</div>' +
    '<div class="marker-line"></div>' +
    '<div>',
    ...markerSettings
  });
}

function getObservationMarker(data, hasSheepCount = false) {
  const markerIcon = markerIcons[data.type];
  const imageIcon = getImageIcon(data);
  let sheepCount = '';
  if (hasSheepCount) {
    const sheepObject = getNumberOfSheepInObservation(data);
    const numberOfSheep = sheepObject.ewes.total + sheepObject.lamb.total;
    sheepCount = '<div class="number-of-sheep">' + numberOfSheep + '</div>';
  }
  return new L.divIcon({
    html: '<div class="marker-container">' +
    '<div class="marker-icon">' +
    '<img src="' + markerIcon.url + '" class="' + markerIcon.className + '" />' +
    '</div>' +
    '<div class="marker-line"></div>' +
    sheepCount +
    imageIcon +
    '<div>',
    ...markerSettings
  });
}

function getCloseIcon(id) {
  return '<div class="close-popup option-container" id="close-popup">' +
      '<i class="fa fa-remove option-container" id="closex' + id + '"></i>' +
      '</div>';
}

function getDeleteIcon(id) {
  return '<div class="delete-observation option-container" id="deletex' + id + '">' +
      '<i class="fa fa-trash"></i>' +
      '</div>';
}

function getObservationHeader(data) {
  console.log(data);
  if (data.type === '' || data.type === undefined) {
    return 'Observasjonen mangler informasjon';
  }
  return data.type;
}

function getButtonContainer(id) {
  return (
      '<div class="button-container">' +
      '<div class="open-observation option-container" id="openx' + id + '">' +
      'Åpne' +
      '</div>' +
      '</div>'
  );
}

function getHtmlForField(label, value) {
  switch (value) {
    case '': {
      return '';
    }
    default: {
      return (
          '<div class="row-container">' +
          '<div class="label">' + label + ':</div>' +
          '<div class="marker-body-text">' + value + '</div>' +
          '</div>'
      );
    }
  }
}

function getSheepBody(data, id) {
  const sheepCountObject = getNumberOfSheepInObservation(data);
  const ewes = sheepCountObject.ewes.total;
  const lamb = sheepCountObject.lamb.total;
  const images = data.images.length;
  let injuryCount = '';
  if (data.count > 0 && data.hasInjury === 'Ja') {
    const html = getHtmlForField('Antall skadde sauer', data.count);
    injuryCount = (
        '<div class="observation-info">' +
        html +
        '</div>'
    );
  }

  return (
      '<div class="sheep-container">' +
      '<div class="counter-container">' +
      '<div class="count-container">' +
      '<div class="icon-container">' +
      '<img class="icon ewe" src="img/vectors/NewObservationForm/ewe_black.svg" alt="sheep icon" />' +
      '</div>' +
      '<div class="icon-text">' + ewes + '</div>' +
      '</div>' +
      '<div class="count-container">' +
      '<div class="icon-container">' +
      '<img class="icon" src="img/vectors/NewObservationForm/lamb_black.svg" alt="sheep icon" />' +
      '</div>' +
      '<div class="icon-text">' + lamb + '</div>' +
      '</div>' +
      '<div class="count-container option-container" id="imagesx' + id + '">' +
      '<div class="icon-container">' +
      '<i class="icon fa fa-image"></i>' +
      '</div>' +
      '<div class="icon-text">' + images + '</div>' +
      '</div>' +
      '</div>' +
      injuryCount +
      getButtonContainer(id) +
      '</div>'
  );
}

function getGenericObservationBody(images, fields, id) {
  let output = '';
  for (let i = 0; i < fields.length; i += 1) {
    output += fields[i];
  }
  return (
      '<div class="generic-observation-container">' +
      '<div class="count-container option-container" id="imagesx' + id + '">' +
      '<div class="icon-container">' +
      '<i class="icon fa fa-image"></i>' +
      '</div>' +
      '<div class="icon-text">' + images + '</div>' +
      '</div>' +
      '<div class="observation-info">' +
      output +
      '</div>' +
      getButtonContainer(id) +
      '</div>'
  );
}

function getDeadSheepBody(data, id) {
  const type = getHtmlForField('Type', data.sheepType);
  const color = getHtmlForField('Farge', data.color);
  const causeOfDeath = getHtmlForField('Dødsårsak', data.causeOfDeath);
  return getGenericObservationBody(data.images.length, [type, color, causeOfDeath], id);
}

function getWoolFoundBody(data, id) {
  const color = getHtmlForField('Farge', data.color);
  return getGenericObservationBody(data.images.length, [color], id);
}

function getDogBody(data, id) {
  const category = getHtmlForField('Kategori', data.category);
  const color = getHtmlForField('Farge', data.dogColor);
  const breed = getHtmlForField('Rase', data.breed);
  return getGenericObservationBody(data.images.length, [category, color, breed], id);
}

function getPredatorBody(data, id) {
  const type = getHtmlForField('Type', data.category);
  const animal = getHtmlForField('Dyr', data.animal);
  return getGenericObservationBody(data.images.length, [type, animal], id);
}

function getReindeerBody(data, id) {
  const count = getHtmlForField('Antall', data.count);
  return getGenericObservationBody(data.images.length, [count], id);
}

export function getMarkerIcon(data) {
  switch (data.type) {
    case 'Sau': {
      return getObservationMarker(data, true);
    }
    case 'Skadet sau':
    case 'Død sau':
    case 'Ullfunn':
    case 'Hund':
    case 'Rovdyr':
    case 'Reinsdyr':
    case 'Annet': {
      return getObservationMarker(data);
    }
    case 'registration': {
      return getUtilityMarker('fa fa-binoculars registration-point-marker');
    }
    case 'home': {
      return getUtilityMarker('fa fa-home observation-point-marker');
    }
    default: {
      return getUtilityMarker('fa fa-question unknown-point-marker');
    }
  }
}

function getObservationBody(data, id) {
  switch (data.type) {
    case 'Sau': {
      return getSheepBody(data, id);
    }
    case 'Død sau': {
      return getDeadSheepBody(data, id);
    }
    case 'Ullfunn': {
      return getWoolFoundBody(data, id);
    }
    case 'Hund': {
      return getDogBody(data, id);
    }
    case 'Rovdyr': {
      return getPredatorBody(data, id);
    }
    case 'Reinsdyr': {
      return getReindeerBody(data, id);
    }
    case 'Annet': {
      return getGenericObservationBody(data.images.length, [], id);
    }
    default: {
      return '<div class="open-observation option-container" id="openx' + id + '">' +
          'Åpne' +
          '</div>';
    }
  }
}

export function getObservationOptionsPopup(id, data) {
  return (
      '<div class="observation-options">' +
      '<div class="header">' +
      getDeleteIcon(id) +
      '<div class="header-text">' + getObservationHeader(data) + '</div>' +
      getCloseIcon(id) +
      '</div>' +
      '<div class="body">' +
      getObservationBody(data, id) +
      '</div>' +
      '</div>'
  );
}