
export function getDistance(lat1, lon1, lat2, lon2) {
  const pi = 0.017453292519943295;    // Math.PI / 180
  const cos = Math.cos;
  const result = 0.5 - cos((lat2 - lat1) * pi) / 2 +
      cos(lat1 * pi) * cos(lat2 * pi) *
      (1 - cos((lon2 - lon1) * pi)) / 2;

  return 12742 * Math.asin(Math.sqrt(result)); // 2 * R; R = 6371 km
}

export function getNumberOfSheepInObservation(data) {
  const ewes = {
    white: 0,
    black: 0,
    blackAndWhite: 0
  };
  const lamb = {
    white: 0,
    black: 0,
    blackAndWhite: 0
  };
  data.groups.forEach(groups => {
    ewes.white += groups.ewes.white;
    ewes.black += groups.ewes.black;
    ewes.blackAndWhite += groups.ewes.blackAndWhite;
    lamb.white += groups.lamb.white;
    lamb.black += groups.lamb.black;
    lamb.blackAndWhite += groups.lamb.blackAndWhite;
  });
  return {
    ewes: {
      ...ewes,
      total: ewes.black + ewes.white + ewes.blackAndWhite
    },
    lamb: {
      ...lamb,
      total: lamb.black + lamb.white + lamb.blackAndWhite
    }
  };
}

export function getNumberOfSheepInObservationRound(file) {
  let countObject = {
    ewes: 0,
    lamb: 0
  };
  Object.keys(file.registrationPoints).forEach(key => {
    const registrationPoint = file.registrationPoints[key];
    Object.keys(registrationPoint.observations).forEach(regKey => {
      const observation = registrationPoint.observations[regKey];
      if (observation.data.type === 'Sau') {
        const observationCount = getNumberOfSheepInObservation(observation.data);
        countObject = {
          ewes: countObject.ewes + observationCount.ewes.total,
          lamb: countObject.lamb + observationCount.lamb.total
        };
      } else if (observation.data.type === 'Skadet sau') {
        if (observation.data.shouldBeCounted === 'Ja') {
          switch (observation.data.sheepType) {
            case 'Søye':
              countObject = {
                ...countObject,
                ewes: countObject.ewes + 1
              };
              break;
            default:
              countObject = {
                ...countObject,
                lamb: countObject.lamb + 1
              };
          }
        }
      }
    });
  });
  return countObject;
}

export function getWalkedDistance(file) {
  let distance = 0;
  const points = file.userTrackingPoints;
  for (let i = 0; i < points.length - 1; i += 1) {
    distance += getDistance(points[i][0], points[i][1], points[i + 1][0], points[i + 1][1]);
  }
  const formattedDistance = Math.round(distance * 1000);
  return formattedDistance + 'm';
}

export function getTimeSpent(file) {
  const createdAt = file.createdTimestamp;
  const lastSaved = file.lastSavedTimestamp;
  let seconds = (lastSaved - createdAt) / 1000;
  const hours = Math.floor(seconds / 3600);
  seconds -= (hours * 3600);
  const minutes = Math.floor(seconds / 60);

  if (hours > 0) {
    return `${hours}t ${minutes}m`;
  }
  return `${minutes}m`;
}

export function getRoundStatistics(round) {
  let injuryCount = 0;
  let deadCount = 0;
  let woolCount = 0;
  let predatorCount = 0;
  let dogCount = 0;
  Object.keys(round.registrationPoints).forEach(key => {
    const registrationPoint = round.registrationPoints[key];
    Object.keys(registrationPoint.observations).forEach(key => {
      const data = registrationPoint.observations[key].data;
      if ('injuries' in data) {
        injuryCount += data.injuries.length;
      }
      if (data.category === 'Død sau') {
        deadCount += 1;
      } else if (data.category === 'Ullfunn') {
        woolCount += 1;
      } else if(data.type === 'Rovdyr') {
        predatorCount += 1;
      } else if(data.type === 'Hund') {
        dogCount += 1;
      }
    })
  });
  return {
    injuryCount,
    deadCount,
    woolCount,
    predatorCount,
    dogCount
  }
}

export function getSheepColor(groups) {
  let color = 'Ukjent';
  Object.keys(groups).forEach(key => {
    console.log(groups[key]);
    if (groups[key].black === 1) {
      color = 'Svart';
    } else if(groups[key].white === 1) {
      color = 'Hvit';
    } else {
      color = 'Svart og hvit';
    }
  });
  return color;
}
